#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/18 13:51:38
'''

from ..utils import sutils
from ..utils import attrs_marker
from ..utils import attrs_manager
from .driver_cache import DriverCache
from ..pylibs.lazy_libs import LazyLibs
from .wechat_minium_proxy import WechatMiniumProxy


class DriverManager(attrs_manager.AttributeManager):

    CHROME = attrs_marker.ConstAttributeMarker("chrome", "谷歌浏览器")
    FIREFOX = attrs_marker.ConstAttributeMarker("firefox", "火狐浏览器")
    EDGE = attrs_marker.ConstAttributeMarker("edge", "Edge浏览器")
    IE = attrs_marker.ConstAttributeMarker("ie", "IE浏览器")
    OPERA = attrs_marker.ConstAttributeMarker("opera", "欧朋（Opera）浏览器")
    SAFARI = attrs_marker.ConstAttributeMarker("safari", "Safari浏览器")
    BLACKBERRY = attrs_marker.ConstAttributeMarker("blackberry", "黑莓")
    PHANTOMJS = attrs_marker.ConstAttributeMarker("phantomjs", "PhantomJS 是一个无界面的webkit内核浏览器")
    ANDROID = attrs_marker.ConstAttributeMarker("android", "安卓")
    WEBKITGTK = attrs_marker.ConstAttributeMarker("webkitgtk", "WebKitGTK")
    SELENIUM_WEBDRIVER_REMOTE = attrs_marker.ConstAttributeMarker("selenium.webdriver.Remote", "调用selenium.webdriver.Remote的名称")
    APPIUM_WEBDRIVER_REMOTE = attrs_marker.ConstAttributeMarker("appium", "调用appium.webdriver.Remote的名称")
    WINDOW_APP = attrs_marker.ConstAttributeMarker("winapp", "window操作系统应用")
    WINDOW_DESKTOP = attrs_marker.ConstAttributeMarker("windesktop", "window操作系统桌面")
    DESKTOP_ALIAS = attrs_marker.AttributeMarker("WindowDesktop", True, "创建的window桌面会话默认别名")
    WECHAT_ALIAS = attrs_marker.AttributeMarker("wechat", True, "微信小程序minium会话默认别名")

    def __init__(self, script_timeout=5.0, implicit_wait_timeout=0.0):
        """

        Args:
            script_timeout: Set the amount of time(seconds) that the script should wait before throwing an error.
            implicit_wait_timeout: Sets a sticky timeout to implicitly wait for an element to be found, or a command to complete.This method only needs to be called one time per session
        @see selenium.webdriver.remote.webdriver.set_script_timeout
        @see selenium.webdriver.remote.webdriver.implicitly_wait
        """

        self.script_timeout = script_timeout
        self.implicit_wait_timeout = implicit_wait_timeout
        self._driver_cache = DriverCache()

    @property
    def index(self):
        """返回当前驱动实例索引"""
        return self._driver_cache.current_index

    @property
    def driver(self):
        """返回当前驱动实例"""

        return self._driver_cache.current_driver

    def open_url(self, url):
        """Loads a web page in the current browser session."""

        return self.driver.get(url)

    def register_driver(self, driver, alias=None, quit_method=None):

        return self._driver_cache.register_driver(driver, alias, quit_method)

    def switch_driver(self, index_or_alias):

        return self._driver_cache.switch_driver(index_or_alias)

    def close_driver(self):
        """关闭当前驱动"""

        return self._driver_cache.close_driver()

    def close_all_drivers(self):

        return self._driver_cache.close_all_drivers()

    def set_script_timeout(self, time_to_wait):
        return self.driver.set_script_timeout(time_to_wait)

    def implicitly_wait(self, time_to_wait):
        return self.driver.implicitly_wait(time_to_wait)

    def create_appdriver(self, alias=None, quit_method=None, script_timeout=None, implicit_wait_timeout=None, *appdriver_args, **appdriver_kwargs):
        """

        Args:
            alias: 缓存中存放驱动实例的别名
            quit_method: appium.webdirver的退出方法名称，不传则默认为quit
            script_timeout: 用于execute_async_script()执行的异步js超时时间
            implicit_wait_timeout: 智能等待超时时间
            appdriver_args: appium.webdirver的位置参数，没有则不需要传
            appdriver_kwargs: appium.webdirver的关键字参数
        See:
            appium.webdriver
        """

        index = self._driver_cache.get_index(alias)
        if index:
            self.switch_driver(alias)
            return index
        driver = LazyLibs.Appium().webdriver.Remote(*appdriver_args, **appdriver_kwargs)
        # print('Opened application with session id %s' % driver.session_id)
        try:
            driver.set_script_timeout(self.script_timeout if script_timeout is None else script_timeout)
        except LazyLibs.Selenium().exceptions.WebDriverException:
            pass
        driver.implicitly_wait(self.implicit_wait_timeout if implicit_wait_timeout is None else implicit_wait_timeout)
        return self.register_driver(driver, alias, quit_method)

    open_app = create_appdriver

    create_win_app_driver = create_appdriver

    def create_webdriver(self, browser=None, alias=None, quit_method=None, script_timeout=None, implicit_wait_timeout=None, *webdriver_args, **webdriver_kwargs):
        """如果在缓存中已存在该别名的驱动实例，则不会创建，而只会把当前驱动实例指向该别名驱动实例，返回该别名驱动实例的索引号

        Args:
            browser: 根据传入的名称调用对应的浏览器驱动
            alias: 缓存中存放驱动实例的别名
            quit_method: appium.webdirver的退出方法名称，不传则默认为quit
            script_timeout: 用于execute_async_script()执行的异步js超时时间
            implicit_wait_timeout: 智能等待超时时间
            webdriver_args:位置参数，没有则不需要传,具体根据browser值确定，如browser为chrome，则为webdirver.Chrome的参数
            webdriver_kwargs: 关键字参数,具体根据browser值确定，如browser为chrome，则为webdirver.Chrome的参数
        See:
            selenium.webdriver.Remote webdirver.Chrome
        """

        wd = LazyLibs.Selenium().webdriver
        index = self._driver_cache.get_index(alias)
        if index:
            self.switch_driver(alias)
            return index
        clazz = self.__class__
        if browser is None:
            browser = clazz.SELENIUM_WEBDRIVER_REMOTE

        classmaps = {
            clazz.CHROME: wd.Chrome,
            clazz.FIREFOX: wd.Firefox,
            clazz.EDGE: wd.Edge,
            clazz.IE: wd.Ie,
            clazz.OPERA: wd.Opera,
            clazz.SAFARI: wd.Safari,
            clazz.BLACKBERRY: wd.BlackBerry,
            clazz.PHANTOMJS: wd.PhantomJS,
            clazz.ANDROID: wd.Android,
            clazz.WEBKITGTK: wd.WebKitGTK,
            clazz.SELENIUM_WEBDRIVER_REMOTE: wd.Remote
        }
        classmaps = {k.lower(): v for k, v in classmaps.items()}
        driverclass = classmaps.get(browser, None)
        if driverclass is None:
            raise ValueError('{} is not a supported browser.'.format(browser))
        driver = driverclass(*webdriver_args, **webdriver_kwargs)
        driver.set_script_timeout(self.script_timeout if script_timeout is None else script_timeout)
        driver.implicitly_wait(self.implicit_wait_timeout if implicit_wait_timeout is None else implicit_wait_timeout)
        return self.register_driver(driver, alias, quit_method)

    def open_browser(self, name, url=None, alias=None, *args, **kwargs):

        webdriver_args = args
        quit_method = kwargs.pop("quit_method", None)
        script_timeout = kwargs.pop("script_timeout", None)
        implicit_wait_timeout = kwargs.pop("implicit_wait_timeout", None)
        webdriver_kwargs = kwargs
        if sutils.is_string(name):
            index = self.create_webdriver(name, alias, quit_method, script_timeout, implicit_wait_timeout, *webdriver_args, **webdriver_kwargs)
            if url is not None:
                self.open_url(url)
            return index
        else:
            raise ValueError('name must be string, please check.')

    def chrome(self, url=None, alias=None, *args, **kwargs):

        k = 'options'
        if k not in kwargs.keys():
            options = LazyLibs.Selenium().webdriver.ChromeOptions()
            options.add_argument('--disable-logging')
            kwargs[k] = options

        return self.open_browser(self.CHROME, url=url, alias=alias, *args, **kwargs)

    def firefox(self, url=None, alias=None, *args, **kwargs):

        return self.open_browser(self.FIREFOX, url=url, alias=alias, *args, **kwargs)

    def ie(self, url=None, alias=None, *args, **kwargs):

        return self.open_browser(self.IE, url=url, alias=alias, *args, **kwargs)

    def edge(self, url=None, alias=None, *args, **kwargs):

        return self.open_browser(self.EDGE, url=url, alias=alias, *args, **kwargs)

    def open_desktop_session(self, remote_url, alias=None):
        """开启window桌面会话"""

        alias = alias if alias else self.DESKTOP_ALIAS
        try:
            self.switch_driver(alias)
        except RuntimeError:
            desktop_capabilities = dict({"app": "Root", "platformName": "Windows", "deviceName": "Windows", "alias": alias, "newCommandTimeout": 3600, "forceMjsonwp": True})
            self.create_win_app_driver(alias=alias, command_executor=remote_url, desired_capabilities=desktop_capabilities)
        return self.index

    def open_window_app(self, remote_url, alias=None, *args, **kwargs):
        """"""

        desired_capabilities = kwargs.get("desired_capabilities", {})
        if "platformName" not in desired_capabilities:
            desired_capabilities["platformName"] = "Windows"
        if "forceMjsonwp" not in desired_capabilities:
            desired_capabilities["forceMjsonwp"] = True
        kwargs["command_executor"] = remote_url
        kwargs["desired_capabilities"] = desired_capabilities
        return self.create_win_app_driver(alias=alias, *args, **kwargs)

    def create_driver(self, driver_name, alias=None, *driver_args, **driver_kwargs):

        clazz = self.__class__
        web_driver_names = [clazz.CHROME, clazz.FIREFOX, clazz.EDGE, clazz.IE, clazz.OPERA, clazz.SAFARI, clazz.BLACKBERRY, clazz.PHANTOMJS, clazz.ANDROID, clazz.WEBKITGTK, clazz.SELENIUM_WEBDRIVER_REMOTE]
        app_driver_names = [clazz.APPIUM_WEBDRIVER_REMOTE]
        win_app_driver_names = [clazz.WINDOW_APP]
        win_desktop_driver_names = [clazz.WINDOW_DESKTOP]

        if driver_name in web_driver_names:
            self.open_browser(driver_name, alias=alias, *driver_args, **driver_kwargs)
        elif driver_name in app_driver_names:
            self.create_appdriver(alias, *driver_args, **driver_kwargs)
        elif driver_name in win_app_driver_names:
            remote_url = driver_kwargs.pop("remote_url", driver_kwargs.get("command_executor", None))
            self.open_window_app(remote_url, alias, *driver_args, **driver_kwargs)
        elif driver_name in win_desktop_driver_names:
            remote_url = driver_kwargs.pop("remote_url", driver_kwargs.get("command_executor", None))
            self.open_desktop_session(driver_kwargs.get(remote_url, None), alias)
        else:
            values = []
            values.extend(web_driver_names)
            values.extend(app_driver_names)
            values.extend(win_app_driver_names)
            values.extend(win_desktop_driver_names)
            raise ValueError('{} is a invalid name,vaild value range is: {}'.format(driver_name, values))

    def open_wechat_minium(self, minium_config=None):

        try:
            mini_proxy = self.switch_driver(self.WECHAT_ALIAS)
        except RuntimeError:
            mini_proxy = WechatMiniumProxy()
            mini_proxy.init_minium(minium_config)
            self.register_driver(mini_proxy, self.WECHAT_ALIAS, "release_minium")
        return mini_proxy

    def switch_to_wechat_minium(self):
        return self.switch_driver(self.WECHAT_ALIAS)


DRIVER_MANAGER = DriverManager()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/17
'''

from unittest.loader import TestLoader
from .abstract_testcase import AbstractTestCase


class SevenTestLoader(TestLoader):

    # 参数命名空间
    args_namespace = None

    def loadTestsFromTestCase(self, testCaseClass):
        if issubclass(testCaseClass, AbstractTestCase):
            loaded_suite = self.suiteClass(testCaseClass.collect_testcases(self.args_namespace))
        else:
            loaded_suite = super().loadTestsFromTestCase(testCaseClass)
        return loaded_suite



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/22 17:23:26
'''

import time
import base64
import inspect
import subprocess

from ..utils import sutils
from ..utils import attrs_manager
from ..utils import attrs_marker
from ..core.errors import WindowNotFound
from ..dm import DRIVER_MANAGER
from ..dm import WIN_APP_DRIVER_HELPER
from ..pylibs.lazy_libs import LazyLibs
from ..utils.screenshot_capturer import ScreenshotCapturer


class AbstractPage(attrs_manager.AttributeManager):
    """ 抽象页 """

    DRIVER_MANAGER = attrs_marker.AttributeMarker(DRIVER_MANAGER, True, "Driver管理器")
    WIN_APP_DRIVER_HELPER = attrs_marker.AttributeMarker(WIN_APP_DRIVER_HELPER, True, "启动和关闭WinAppDriver.exe助手")

    def __init__(self, driver=None, alias=None, timeout=0.0, *args, **kwargs):
        """一般不建议创建页面的时候直接创建驱动实例，建议页面实例化后再调用页面提供的相关驱动实例化方法更好

        Args:
            driver: 驱动实例或者驱动名称
            alias: 在缓存中驱动实例的别名，可通过该别名来切换驱动实例，如果在缓存中已存在则直接切换到该实例
            timeout: 查找元素的默认超时时间
            args: 见DriverManager.create_driver
            kwargs: 见DriverManager.create_driver
        """

        self._dm = self.__class__.DRIVER_MANAGER
        self._dm.script_timeout = kwargs.pop("script_timeout", 5.0)
        self._dm.implicit_wait_timeout = kwargs.pop("implicit_wait_timeout", 0.0)

        if isinstance(driver, str):
            self.create_driver(driver, alias, *args, **kwargs)
        elif driver:
            self._dm.register_driver(driver, alias)

        self.timeout = timeout
        self._build_elements()
        self._build_actions()
        self.init()

    def init(self):

        pass

    def _build_elements(self):

        self.elements = self.__class__.Elements(self)

    def _build_actions(self):

        self.actions = self.__class__.Actions(self)

    @property
    def browser(self):

        return self.driver

    @property
    def driver(self):

        return self._dm.driver

    @property
    def window_app(self):

        return self._dm.driver

    @property
    def action_chains(self):
        """动作链对象"""

        return LazyLibs.Selenium().ActionChains(self.driver)

    @property
    def driver_manager(self):
        """Driver管理器"""

        return self._dm

    @property
    def index(self):
        """driver索引"""

        return self._dm.index

    @classmethod
    def startup_winappdriver(cls, executable_path, output_stream=None, auto_close_output_stream=False):
        """启动WinAppDriver.exe

        Args:
            executable_path: WinAppDriver.exe程序完整路径
            output_stream: 输出流
            auto_close_output_stream: 是否自动关闭output_stream
        """

        return cls.WIN_APP_DRIVER_HELPER.startup_winappdriver(executable_path, output_stream, auto_close_output_stream)

    @classmethod
    def shutdown_winappdriver(cls):
        """关闭WinAppDriver.exe"""

        return cls.WIN_APP_DRIVER_HELPER.shutdown_winappdriver()

    def open_url(self, url):

        self.driver_manager.open_url(url)
        return self

    def open_app(self, remote_url='http://127.0.0.1:4444/wd/hub', alias=None, **kwargs):

        kwargs['implicit_wait_timeout'] = kwargs.get('implicit_wait_timeout', 7.0)
        kwargs['command_executor'] = remote_url
        self._dm.open_app(alias=alias, **kwargs)
        return self

    def open_window_app(self, remote_url="http://127.0.0.1:4723", desired_capabilities={}, alias=None, window_name=None, splash_delay=0, exact_match=True, desktop_alias=None):
        """ 创建 Windows 应用程序驱动程序会话

        Args:
            remote_url: WinAppDriver or Appium server url
            desired_capabilities:用于创建 Windows 应用程序驱动程序会话的功能
                app Application identifier or executable full path Microsoft.MicrosoftEdge_8wekyb3d8bbwe!MicrosoftEdge

                appArguments Application launch arguments https://github.com/Microsoft/WinAppDriver

                appTopLevelWindow Existing application top level window to attach to 0xB822E2

                appWorkingDir 应用程序工作目录 (仅限经典应用程序) C:\\Temp

                platformName Target platform name Windows

                platformVersion Target platform version 1.0

            alias: 为创建的应用会话设置别名
            window_name: 要附加的窗口名称，通常在启动屏幕之后
            exact_match: 如果窗口名称不需要完全匹配，则设置为False
            desktop_alias: 为创建的桌面会话设置别名，将默认为“WindowDesktop”
        """

        if window_name:
            subprocess.Popen(desired_capabilities['app'])
            if splash_delay > 0:
                # print('Waiting %s seconds for splash screen' % splash_delay)
                self.sleep(splash_delay)
            return self.switch_window_app_by_name(remote_url, alias=alias, window_name=window_name, exact_match=exact_match, desktop_alias=desktop_alias, **desired_capabilities)
        self.driver_manager.open_desktop_session(remote_url, desktop_alias)
        self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_capabilities)
        return self

    def switch_window_app_by_window_element(self, remote_url, window_element, alias=None, **kwargs):

        desired_caps = kwargs
        window_name = window_element.get_attribute("Name")
        if not window_name:
            msg = 'Error connecting webdriver to window "' + window_name + '". \n'
        else:
            msg = 'Error connecting webdriver to window(which window element tag name is:{}). \n'.format(window_element.tag_name)
        window = hex(int(window_element.get_attribute("NativeWindowHandle")))
        if "app" in desired_caps:
            del desired_caps["app"]
        if "platformName" not in desired_caps:
            desired_caps["platformName"] = "Windows"
        if "forceMjsonwp" not in desired_caps:
            desired_caps["forceMjsonwp"] = True
        desired_caps["appTopLevelWindow"] = window
        try:
            self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_caps)
        except Exception as e:
            raise WindowNotFound(msg + str(e))
        return self

    def switch_window_app_by_name(self, remote_url, window_name, alias=None, timeout=5, exact_match=True, desktop_alias=None, **kwargs):

        desired_caps = kwargs
        self.driver_manager.open_desktop_session(remote_url, desktop_alias)
        window_xpath = '//Window[contains(@Name, "' + window_name + '")]'
        window_locator = window_name
        try:
            if exact_match:
                window = self.find_element_by_name(window_locator)
            else:
                window = self.find_element_by_xpath(window_xpath)
            # print('Window_name "%s" found.' % window_name)
            window = hex(int(window.get_attribute("NativeWindowHandle")))
        except Exception:
            try:
                if exact_match:
                    window = self.find_element_by_name(window_locator, timeout=timeout)
                else:
                    window = self.find_element_by_xpath(window_xpath, timeout=timeout)
                # print('Window_name "%s" found.' % window_name)
                window = hex(int(window.get_attribute("NativeWindowHandle")))
            except Exception as e:
                # print('Closing desktop session.')
                raise LazyLibs.Selenium().exceptions.NoSuchWindowException('Error finding window "' + window_name + '" in the desktop session. ' 'Is it a top level window handle?' + '. \n' + str(e))
        if "app" in desired_caps:
            del desired_caps["app"]
        if "platformName" not in desired_caps:
            desired_caps["platformName"] = "Windows"
        if "forceMjsonwp" not in desired_caps:
            desired_caps["forceMjsonwp"] = True
        desired_caps["appTopLevelWindow"] = window
        # global application
        try:
            # print('Connecting to window_name "%s".' % window_name)
            self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_caps)
        except Exception as e:
            raise WindowNotFound('Error connecting webdriver to window "' + window_name + '". \n' + str(e))
        return self

    def create_driver(self, driver_name, alias=None, *driver_args, **driver_kwargs):

        return self._dm.create_driver(driver_name, alias=alias, *driver_args, **driver_kwargs)

    def close_driver(self):
        self._dm.close_driver()
        return self

    def close_all_drivers(self):
        self._dm.close_all_drivers()
        return self

    def open_browser(self, browser_name, url=None, alias=None, *args, **kwargs):

        self._dm.open_browser(browser_name, url, alias, *args, **kwargs)
        return self

    def ie(self, url=None, alias=None, *args, **kwargs):

        self.driver_manager.ie(url=url, alias=alias, *args, **kwargs)
        return self

    def chrome(self, url=None, alias=None, **kwargs):
        """
        Creates a new instance of the chrome driver.

        Starts the service and then creates new instance of chrome driver.

        :Args:
           kwargs:
            - executable_path - path to the executable. If the default is used it assumes the executable is in the $PATH
            - port - port you would like the service to run, if left as 0, a free port will be found.
            - options - this takes an instance of ChromeOptions
            - service_args - List of args to pass to the driver service
            - desired_capabilities - Dictionary object with non-browser specific
            capabilities only, such as "proxy" or "loggingPref".
            - service_log_path - Where to log information from the driver.
            - chrome_options - Deprecated argument for options
            - keep_alive - Whether to configure ChromeRemoteConnection to use HTTP keep-alive.
        """
        args = ()
        self.driver_manager.chrome(url=url, alias=alias, *args, **kwargs)
        return self

    def firefox(self, url=None, alias=None, *args, **kwargs):

        self.driver_manager.firefox(url=url, alias=alias, *args, **kwargs)
        return self

    def switch_driver(self, index_or_alias):
        self._dm.switch_driver(index_or_alias)
        return self

    switch_browser = switch_driver
    switch_window_app = switch_driver

    @staticmethod
    def is_web_element(obj):

        return isinstance(obj, LazyLibs.Selenium().WebElement)

    def _validate_timeout(self, timeout):

        return isinstance(timeout, (int, float))

    def maximize_window(self):

        self.driver.maximize_window()
        return self

    def minimize_window(self):

        self.driver.minimize_window()
        return self

    def find_element_by_id(self, element_id, timeout=None, parent=None):
        """Finds an element by id.

        @param element_id - The id of the element to be found.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage  element = driver.find_element_by_id('foo')
        """

        return self.find_element(by=LazyLibs.Selenium().By.ID, locator=element_id, timeout=timeout, parent=parent)

    def find_elements_by_id(self, element_id, timeout=None, parent=None):
        """
        Finds multiple elements by id.

        @param element_id - The id of the elements to be found.

        @return list of WebElement - a list with elements if any was found.  An empty list if not

        @usage  elements = driver.find_elements_by_id('foo')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.ID, locator=element_id, timeout=timeout, parent=parent)

    def find_element_by_xpath(self, xpath, timeout=None, parent=None):
        """
        Finds an element by xpath.

        @param xpath - The xpath locator of the element to find.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage  element = driver.find_element_by_xpath('//div/td[1]')
        """

        return self.find_element(by=LazyLibs.Selenium().By.XPATH, locator=xpath, timeout=timeout, parent=parent)

    def find_elements_by_xpath(self, xpath, timeout=None, parent=None):
        """
        Finds multiple elements by xpath.

        @param xpath - The xpath locator of the elements to be found.

        @return list of WebElement - a list with elements if any was found.  An empty list if not

        @usage  elements = driver.find_elements_by_xpath("//div[contains(@class, 'foo')]")
        """

        return self.find_elements(by=LazyLibs.Selenium().By.XPATH, locator=xpath, timeout=timeout, parent=parent)

    def find_element_by_link_text(self, link_text, timeout=None, parent=None):
        """
        Finds an element by link text.

        @param link_text: The text of the element to be found.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage  element = driver.find_element_by_link_text('Sign In')
        """

        return self.find_element(by=LazyLibs.Selenium().By.LINK_TEXT, locator=link_text, timeout=timeout, parent=parent)

    def find_elements_by_link_text(self, link_text, timeout=None, parent=None):
        """
        Finds elements by link text.

        @param link_text: The text of the elements to be found.

        @return list of webelement - a list with elements if any was found.  an empty list if not

        @usage  elements = driver.find_elements_by_link_text('Sign In')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.LINK_TEXT, locator=link_text, timeout=timeout, parent=parent)

    def find_element_by_partial_link_text(self, link_text, timeout=None, parent=None):
        """
        Finds an element by a partial match of its link text.

        @param link_text The text of the element to partially match on.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage element = driver.find_element_by_partial_link_text('Sign')
        """

        return self.find_element(by=LazyLibs.Selenium().By.PARTIAL_LINK_TEXT, locator=link_text, timeout=timeout, parent=parent)

    def find_elements_by_partial_link_text(self, link_text, timeout=None, parent=None):
        """
        Finds elements by a partial match of their link text.

        @param link_text: The text of the element to partial match on.

        @return list of webelement - a list with elements if any was found.  an empty list if not

        @usage:
            elements = driver.find_elements_by_partial_link_text('Sign')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.PARTIAL_LINK_TEXT, locator=link_text, timeout=timeout, parent=parent)

    def find_element_by_name(self, name, timeout=None, parent=None):
        """
        Finds an element by name.

        @param name: The name of the element to find.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage
            element = driver.find_element_by_name('foo')
        """

        return self.find_element(by=LazyLibs.Selenium().By.NAME, locator=name, timeout=timeout, parent=parent)

    def find_elements_by_name(self, name, timeout=None, parent=None):
        """
        Finds elements by name.

        @param name: The name of the elements to find.

        @return list of webelement - a list with elements if any was found.  an empty list if not

        @usage:
            elements = driver.find_elements_by_name('foo')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.NAME, locator=name, timeout=timeout, parent=parent)

    def find_element_by_tag_name(self, name, timeout=None, parent=None):
        """
        Finds an element by tag name.

        @param name - name of html tag (eg: h1, a, span)

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage
            element = driver.find_element_by_tag_name('h1')
        """

        return self.find_element(by=LazyLibs.Selenium().By.TAG_NAME, locator=name, timeout=timeout, parent=parent)

    def find_elements_by_tag_name(self, name, timeout=None, parent=None):
        """
        Finds elements by tag name.

        @param name - name of html tag (eg: h1, a, span)

        @return list of WebElement - a list with elements if any was found.  An empty list if not

        @usage
            elements = driver.find_elements_by_tag_name('h1')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.TAG_NAME, locator=name, timeout=timeout, parent=parent)

    def find_element_by_class_name(self, name, timeout=None, parent=None):
        """
        Finds an element by class name.

        @param name The class name of the element to find.

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage:
            element = driver.find_element_by_class_name('foo')
        """

        return self.find_element(by=LazyLibs.Selenium().By.CLASS_NAME, locator=name, timeout=timeout, parent=parent)

    def find_elements_by_class_name(self, name, timeout=None, parent=None):
        """
        Finds elements by class name.

        @param name The class name of the elements to find.

        @return list of WebElement - a list with elements if any was found.  An empty list if not

        @usage
            elements = driver.find_elements_by_class_name('foo')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.CLASS_NAME, locator=name, timeout=timeout, parent=parent)

    def find_element_by_css_selector(self, css_selector, timeout=None, parent=None):
        """
        Finds an element by css selector.

        @param css_selector - CSS selector string, ex: 'a.nav#home'

        @return WebElement - the element if it was found

        @raise NoSuchElementException - if the element wasn't found

        @usage
            element = driver.find_element_by_css_selector('#foo')
        """

        return self.find_element(by=LazyLibs.Selenium().By.CSS_SELECTOR, locator=css_selector, timeout=timeout, parent=parent)

    def find_elements_by_css_selector(self, css_selector, timeout=None, parent=None):
        """
        Finds elements by css selector.

        @param css_selector - CSS selector string, ex: 'a.nav#home'

        @return list of WebElement - a list with elements if any was found.  An empty list if not

        @usage
            elements = driver.find_elements_by_css_selector('.foo')
        """

        return self.find_elements(by=LazyLibs.Selenium().By.CSS_SELECTOR, locator=css_selector, timeout=timeout, parent=parent)

    def find_element(self, by="id", locator=None, timeout=None, parent=None):
        """查找匹配的元素

        Args:
            - by       - 查找方式
            - locator  - 元素定位器
            - timeout  - 查找元素超时时间
            - parent   - 父元素,提供则从父元素下查找

        Raises:
            - NoSuchElementException - if the element wasn't found
            - TimeoutException - if the element wasn't found when time out

        Returns:
            - WebElement - the element if it was found
        """

        if not (timeout and self._validate_timeout(timeout)):
            timeout = self.timeout

        if parent and self.is_web_element(parent):
            driver = parent.parent
        else:
            driver = self.browser

        # 如果设置的超时时间无效或者超时时间小于0，则不会执行超时
        if not (self._validate_timeout(timeout) and timeout > 0):
            return driver.find_element(by, locator)
        message = "{} with locator '{}' not found".format(by, locator)
        try:
            element = LazyLibs.Selenium().WebDriverWait(driver, timeout).until(lambda x: x.find_element(by, locator))
        except LazyLibs.Selenium().exceptions.TimeoutException as exc:
            message = message + "in {timeout}".format(timeout=timeout)
            screen = getattr(exc, 'screen', None)
            stacktrace = getattr(exc, 'stacktrace', None)
            raise LazyLibs.Selenium().exceptions.TimeoutException(message, screen, stacktrace)
        except LazyLibs.Selenium().exceptions.NoSuchWindowException as e:
            message = message + "," + e.msg
            screen = getattr(e, 'screen', None)
            stacktrace = getattr(e, 'stacktrace', None)
            raise LazyLibs.Selenium().exceptions.NoSuchWindowException(message, screen, stacktrace)
        except Exception as e:
            raise e
        else:
            return element

    def find_elements(self, by="id", locator=None, timeout=None, parent=None):
        """查找所有匹配的元素

        Args:
            - by - 查找方式
            - locator - 元素定位器
            - timeout - 查找元素超时时间
            - parent - 父元素,提供则从父元素下查找

        Returns:
            - list of WebElement - a list with elements if any was found. An empty list if not
        """

        if not (timeout and self._validate_timeout(timeout)):
            timeout = self.timeout

        if parent and self.is_web_element(parent):
            driver = parent.parent
        else:
            driver = self.browser
        # 如果设置的超时时间无效或者超时时间小于0，则不会执行超时
        if not (self._validate_timeout(timeout) and timeout > 0):
            return driver.find_elements(by, locator)
        message = "{} with locator '{}' not found.".format(by, locator)
        try:
            elements = LazyLibs.Selenium().WebDriverWait(driver, timeout).until(lambda x: x.find_elements(by, locator))
        except LazyLibs.Selenium().exceptions.TimeoutException as t:
            message = message + "in {timeout}".format(timeout=timeout)
            screen = getattr(t, 'screen', None)
            stacktrace = getattr(t, 'stacktrace', None)
            # raise TimeoutException(message, screen, stacktrace)
            # print(message)
            return []
        except LazyLibs.Selenium().exceptions.NoSuchWindowException as e:
            message = message + "," + e.message
            screen = getattr(e, 'screen', None)
            stacktrace = getattr(e, 'stacktrace', None)
            raise LazyLibs.Selenium().exceptions.NoSuchWindowException(message, screen, stacktrace)
        except Exception as e:
            raise e
        else:
            return elements

    def find_element_by_ios_uiautomation(self, uia_string, timeout=None):
        """Finds an element by uiautomation in iOS.

        Args:
            uia_string (str): The element name in the iOS UIAutomation library

        Usage:
            driver.find_element_by_ios_uiautomation('.elements()[1].cells()[2]')

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.IOS_UIAUTOMATION, locator=uia_string, timeout=timeout, parent=None)

    def find_elements_by_ios_uiautomation(self, uia_string, timeout=None):
        """Finds elements by uiautomation in iOS.

        Args:
            uia_string: The element name in the iOS UIAutomation library

        Usage:
            driver.find_elements_by_ios_uiautomation('.elements()[1].cells()[2]')

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.IOS_UIAUTOMATION, locator=uia_string, timeout=timeout, parent=None)

    def find_element_by_ios_predicate(self, predicate_string, timeout=None):
        """Find an element by ios predicate string.

        Args:
            predicate_string (str): The predicate string

        Usage:
            driver.find_element_by_ios_predicate('label == "myLabel"')

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.IOS_PREDICATE, locator=predicate_string, timeout=timeout, parent=None)

    def find_elements_by_ios_predicate(self, predicate_string, timeout=None):
        """Finds elements by ios predicate string.

        Args:
            predicate_string (str): The predicate string

        Usage:
            driver.find_elements_by_ios_predicate('label == "myLabel"')

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.IOS_PREDICATE, locator=predicate_string, timeout=timeout, parent=None)

    def find_element_by_ios_class_chain(self, class_chain_string, timeout=None):
        """Find an element by ios class chain string.

        Args:
            class_chain_string (str): The class chain string

        Usage:
            driver.find_element_by_ios_class_chain('XCUIElementTypeWindow/XCUIElementTypeButton[3]')

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.IOS_CLASS_CHAIN, locator=class_chain_string, timeout=timeout, parent=None)

    def find_elements_by_ios_class_chain(self, class_chain_string, timeout=None):
        """Finds elements by ios class chain string.

        Args:
            class_chain_string (str): The class chain string

        Usage:
            driver.find_elements_by_ios_class_chain('XCUIElementTypeWindow[2]/XCUIElementTypeAny[-2]')

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.IOS_CLASS_CHAIN, locator=class_chain_string, timeout=timeout, parent=None)

    def find_element_by_android_uiautomator(self, uia_string, timeout=None):

        return self.find_element(by=LazyLibs.Appium().MobileBy.ANDROID_UIAUTOMATOR, locator=uia_string, timeout=timeout, parent=None)

    def find_elements_by_android_uiautomator(self, uia_string, timeout=None):

        return self.find_elements(by=LazyLibs.Appium().MobileBy.ANDROID_UIAUTOMATOR, locator=uia_string, timeout=timeout, parent=None)

    def find_element_by_android_viewtag(self, tag, timeout=None):
        """Finds element by [View#tags](https://developer.android.com/reference/android/view/View#tags) in Android.

        It works with [Espresso Driver](https://github.com/appium/appium-espresso-driver).

        Args:
            tag (str): The tag name of the view to look for

        Usage:
            driver.find_element_by_android_viewtag('a tag name')

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.ANDROID_VIEWTAG, locator=tag, timeout=timeout, parent=None)

    def find_elements_by_android_viewtag(self, tag, timeout=None):
        """Finds element by [View#tags](https://developer.android.com/reference/android/view/View#tags) in Android.

        It works with [Espresso Driver](https://github.com/appium/appium-espresso-driver).

        Args:
            tag (str): The tag name of the view to look for

        Usage:
            driver.find_elements_by_android_viewtag('a tag name')

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.ANDROID_VIEWTAG, locator=tag, timeout=timeout, parent=None)

    def find_element_by_image(self, img_path, timeout=None):
        """Finds a portion of a screenshot by an image.

        Uses driver.find_image_occurrence under the hood.

        Args:
            img_path (str): a string corresponding to the path of a image

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        with open(img_path, 'rb') as i_file:
            b64_data = base64.b64encode(i_file.read()).decode('UTF-8')

        return self.find_element(by=LazyLibs.Appium().MobileBy.IMAGE, locator=b64_data, timeout=timeout, parent=None)

    def find_elements_by_image(self, img_path, timeout=None):
        """Finds a portion of a screenshot by an image.

        Uses driver.find_image_occurrence under the hood. Note that this will
        only ever return at most one element

        Args:
            img_path (str): a string corresponding to the path of a image

        Return:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        with open(img_path, 'rb') as i_file:
            b64_data = base64.b64encode(i_file.read()).decode('UTF-8')

        return self.find_elements(by=LazyLibs.Appium().MobileBy.IMAGE, locator=b64_data, timeout=timeout, parent=None)

    def find_element_by_accessibility_id(self, accessibility_id, timeout=None):
        """Finds an element by accessibility id.

        Args:
            accessibility_id (str): A string corresponding to a recursive element search using the
                Id/Name that the native Accessibility options utilize

        Usage:
            driver.find_element_by_accessibility_id()

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.ACCESSIBILITY_ID, locator=accessibility_id, timeout=timeout, parent=None)

    def find_elements_by_accessibility_id(self, accessibility_id, timeout=None):
        """Finds elements by accessibility id.

        Args:
            accessibility_id (str): a string corresponding to a recursive element search using the
                Id/Name that the native Accessibility options utilize

        Usage:
            driver.find_elements_by_accessibility_id()

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.ACCESSIBILITY_ID, locator=accessibility_id, timeout=timeout, parent=None)

    def find_element_by_custom(self, selector, timeout=None):
        """Finds an element in conjunction with a custom element finding plugin

        Args:
            selector (str): a string of the form "module:selector", where "module" is
                the shortcut name given in the customFindModules capability, and
                "selector" is the string that will be passed to the custom element
                finding plugin itself

        Usage:
            driver.find_element_by_custom("foo:bar")

        Returns:
            `appium.webdriver.webelement.WebElement`

        :rtype: `MobileWebElement`
        """
        return self.find_element(by=LazyLibs.Appium().MobileBy.CUSTOM, locator=selector, timeout=timeout, parent=None)

    def find_elements_by_custom(self, selector, timeout=None):
        """Finds elements in conjunction with a custom element finding plugin

        Args:
            selector: a string of the form "module:selector", where "module" is
                the shortcut name given in the customFindModules capability, and
                "selector" is the string that will be passed to the custom element
                finding plugin itself

        Usage:
            driver.find_elements_by_custom("foo:bar")

        Returns:
            :obj:`list` of :obj:`appium.webdriver.webelement.WebElement`

        :rtype: list of `MobileWebElement`
        """
        return self.find_elements(by=LazyLibs.Appium().MobileBy.CUSTOM, locator=selector, timeout=timeout, parent=None)

    def sleep(self, seconds):
        """seconds the length of time to sleep in seconds"""
        time.sleep(seconds)
        return self

    def wait_until_contains(self, text, timeout=None):
        """等待文本出现在当前页面"""

        locator = "//*contains(., %s)" % sutils.escape_xpath_value(text)
        try:
            self.find_element_by_xpath(locator, timeout=timeout)
        except Exception:
            message = "Text '%s' did not appear in %s." % (text, timeout)
            raise LazyLibs.Selenium().exceptions.TimeoutException(message)

    @classmethod
    def wait_until(cls, callable_method, message="", timeout=None, poll_frequency=0.2, ignored_exceptions=None):

        if timeout is None:
            timeout = 0.0
        exceptions = [LazyLibs.Selenium().exceptions.NoSuchElementException]
        if ignored_exceptions is not None:
            try:
                exceptions.extend(iter(ignored_exceptions))
            except TypeError:
                exceptions.append(ignored_exceptions)
        _ignored_exceptions = tuple(exceptions)
        end_time = time.time() + timeout
        not_found = None
        while True:
            try:
                if callable_method():
                    return
            except _ignored_exceptions as err:
                not_found = str(err)
            else:
                not_found = None
            if time.time() > end_time:
                break
            time.sleep(poll_frequency)
        raise LazyLibs.Selenium().exceptions.TimeoutException(not_found or message)

    def execute_script(self, script, *args):

        return self.driver.execute_script(script, *args)

    def click_element_by_javascript(self, web_element):

        script = "arguments[0].click();"
        return self.execute_script(script, web_element)

    def scroll_to(self, xpos, ypos):
        """scroll to any position of an opened window of browser"""

        js_code = "window.scrollTo(%s, %s);" % (xpos, ypos)
        self.execute_script(js_code)

    def scroll_into_view(self, web_element):

        js_code = 'arguments[0].scrollIntoView();'
        self.execute_script(js_code, web_element)

    def scroll_to_bottom(self):

        bottom = self.execute_script("return document.body.scrollHeight;")
        self.scroll_to(0, bottom)

    def scroll_to_top(self):

        self.scroll_to(0, 0)

    def drag_and_drop(self, source, target):

        self.action_chains.drag_and_drop(source, target).perform()

    def screenshot(self, file_name):
        """截图并保存

        Args:
            file_name: 保存截图文件的完整路径名
        Usage:
            page.screenshot("E:\\SevenPytest\\screenshots\\debug.png")
        """
        return ScreenshotCapturer.screenshot(file_name, self.driver)

    @property
    def session_id(self):
        """Returns the currently active browser session id"""

        return self.driver.session_id

    @property
    def page_source(self):
        """Returns the entire HTML source of the current page or frame."""

        return self.driver.page_source

    @property
    def title(self):
        """Returns the title of the current page."""

        return self.driver.title

    @property
    def active_element(self):
        """Returns the element with focus, or BODY if nothing has focus.

        @see selenium.webdriver.remote.switch_to.SwitchTo.active_element
        @usage element = page.active_element
        """
        return self.driver.switch_to.active_element

    @property
    def alert(self):
        """Switches focus to an alert on the page.

        @see selenium.webdriver.remote.switch_to.SwitchTo.alert
        @usage alert = page.alert
        """
        return self.driver.switch_to.alert

    def select_frame(self, reference):
        """切换frame

        @param reference frame id name index 或 webelement 对象
        @see selenium.webdriver.remote.switch_to.SwitchTo.frame()
        @usage page.select_frame()
        """
        self.driver.switch_to.frame(reference)

    def default_frame(self):
        """ Switch focus to the default frame.

        @see selenium.webdriver.remote.switch_to.SwitchTo.default_content()
        @usage page.default_frame()
        """
        self.driver.switch_to.default_content()
        return self

    def parent_frame(self):
        """嵌套frame时，可以从子frame切回父frame

        @see selenium.webdriver.remote.switch_to.SwitchTo.parent_frame()
        @usage page.parent_frame()
        """

        self.driver.switch_to.parent_frame()

    @property
    def current_window_handle(self):

        return self.driver.current_window_handle

    @property
    def window_handles(self):

        return self.driver.window_handles

    def switch_window(self, window_name_or_handle):
        """切换浏览器窗口

        @param window_name_or_handle 窗口句柄或窗口名
        """
        self.driver.switch_to.window(window_name_or_handle)

    def switch_current_window(self):
        """切换到当前浏览器"""

        self.switch_window(self.current_window_handle)
        return self

    @property
    def window_infos_maps(self):
        """所有窗口信息

        @return 返回列表 [{"handle":window handle, "name": window name, "title": window title, "url": window url}, ...]
        """
        infos_maps = []
        try:
            source_handle = self.current_window_handle
        except LazyLibs.Selenium().exceptions.NoSuchWindowException:
            source_handle = None
        try:
            for handle in self.window_handles:
                self.driver.switch_to.window(handle)
                infos_maps.append(self._get_window_infos_map(handle))
        finally:
            if source_handle:
                self.driver.switch_to.window(source_handle)
        return infos_maps

    def _get_window_infos_map(self, handle):

        title = self.driver.title
        try:
            name = self.execute_script("return window.name;")
        except Exception:
            name = title
        try:
            current_url = self.driver.current_url
        except Exception:
            current_url = None

        infos = {"handle": handle, "name": name, "title": title, "url": current_url}
        return infos

    def switch_window_by_title(self, title, matcher=None, timeout=None):
        """根据标题切换窗口

        Args:
            title: 窗口标题
            matcher: 匹配函数，接收两个参数，遍历传入每一个窗口的标题给第一个参数，要打开的窗口标题传给第二个参数，匹配返回True否则返回False
            timeout: 超时时间
        """
        def _switch_window_by_title():
            be_found = False
            for win_handle in self.window_handles:
                self.switch_window(win_handle)
                info = self._get_window_infos_map(win_handle)
                if inspect.isfunction(matcher):
                    if matcher(info["title"], title):
                        be_found = True
                        break
                else:
                    if info["title"] == title:
                        be_found = True
                        break
            return be_found

        message = "No window matching title(%s)" % title
        self.wait_until(_switch_window_by_title, message=message, timeout=timeout)
        return self

    def switch_window_by_url(self, url, matcher=None, timeout=None):
        """根据url切换窗口

        @see select_window_by_url(self, url, matcher=None)
        """
        def _switch_window_by_url():
            be_found = False
            for win_handle in self.window_handles:
                self.switch_window(win_handle)
                info = self._get_window_infos_map(win_handle)
                if inspect.isfunction(matcher):
                    if matcher(info["url"], url):
                        be_found = True
                        break
                else:
                    if info["url"] == url:
                        be_found = True
                        break
            return be_found

        message = "No window matching url(%s)" % url
        self.wait_until(_switch_window_by_url, message=message, timeout=timeout)
        return self

    def set_window_size(self, width, height, window_handle='current'):
        """Sets current windows size to given ``width`` and ``height``

        @see WebDriver.set_window_size(self, width, height, windowHandle='current')
        """

        self.driver.set_window_size(width, height, window_handle)
        return self

    def get_window_size(self, window_handle='current'):
        """Gets the width and height of the current window.

        @see WebDriver.get_window_size(self, windowHandle='current')
        """
        return self.driver.get_window_size(window_handle)

    def set_window_position(self, x, y, window_handle='current'):

        self.driver.set_window_position(self, x, y, window_handle)
        return self

    def get_window_position(self, window_handle='current'):

        return self.driver.get_window_position(window_handle)

    def refresh(self):
        """刷新当前页面"""

        self.browser.refresh()
        return self

    def hide_keyboard(self, key_name=None, key=None, strategy=None):
        self.driver.hide_keyboard(key_name=key_name, key=key, strategy=strategy)
        return self

    def keyevent(self, keycode, metastate=None):
        """Sends a keycode to the android device.

        @see appium.webdriver.exceptions.keyboard.Keyboard.keyevent
        """
        self.driver.keyevent(keycode, metastate=metastate)
        return self

    def press_keycode(self, keycode, metastate=None, flags=None):
        """Sends a keycode to the device.

        Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html
        @see appium.webdriver.exceptions.keyboard.Keyboard.press_keycode
        """
        self.driver.press_keycode(keycode, metastate=metastate, flags=flags)
        return self

    def create_select_element_wrapper(self, select_element):
        """创建操作html select 元素的包装器

        Args:
            select_element: select标签元素

        Returns: 返回 Select 实例
            提供以下属性：
                options 返回select元素的所有选项
                all_selected_options 返回所有被选中的选项
                first_selected_option 第一个选中项

            提供以下方法：
                select_by_value(value) 通过选项中的value属性值选中选项
                select_by_index(index)
                select_by_visible_text(text) 通过选项中的文本选中选项 - <option>text</option>

                deselect_all()
                deselect_by_value(value)
                deselect_by_index(index)
                deselect_by_visible_text(text)

        See: selenium.webdriver.support.ui.Select
        """

        return LazyLibs.Selenium().Select(select_element)

    def close_current_window(self):
        """ Closes the current window.

        @see selenium.webdriver.remote.webdriver.close()
        """
        self.driver.close()

    def is_element_enabled(self, element):
        """判断元素是否可用"""

        return (element.is_enabled() and element.get_attribute("readonly") is None)

    def get_value(self, element):

        return element.get_attribute("value")

    @classmethod
    def raise_no_such_element_exc(cls, message):

        raise LazyLibs.Selenium().exceptions.NoSuchElementException(message)

    @classmethod
    def join_xpaths(cls, *xpaths):
        def join_two_xpath(x1, x2):
            """拼接xpath"""

            slash = '/'
            if x2.strip() == "":
                return x1
            if x1.endswith(slash) and x2.startswith(slash):
                return x1 + x2[1:]
            elif not x1.endswith(slash) and x2.startswith(slash):
                return x1 + x2
            elif x1.endswith(slash) and not x2.startswith(slash):
                return x1 + x2
            else:
                return x1 + slash + x2

        first = xpaths[0]
        others = xpaths[1:]
        full_xpath = first
        for one in others:
            full_xpath = join_two_xpath(full_xpath, one)
        return full_xpath

    class Elements(object):
        def __init__(self, page):

            self.page = page

        def sleep(self, seconds):
            """延时"""

            self.page.sleep(seconds)
            return self

    class Actions(object):
        def __init__(self, page):

            self.page = page

        def sleep(self, seconds):
            """延时"""

            self.page.sleep(seconds)
            return self

        def turn_to_page(self, page_number):
            """翻页， 由具体页面实现"""

            raise NotImplementedError



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/04/06
'''

import os
import base64
import warnings
import datetime
import tempfile

from ..pylibs.lazy_libs import LazyLibs


class WechatMiniumProxy(object):

    DEVICE_INFO = {}
    DEFAULT_MINI_CONFIG = {
        "platform": "ide",
        "debug_mode": "info",
        "close_ide": False,
        "no_assert_capture": False,
        "auto_relaunch": False,
        "device_desire": {},
        "report_usage": True,
        "remote_connect_timeout": 180,
        "use_push": True
    }

    def __init__(self):

        self.mini = None
        self.native = None
        self.log_message_list = []

    def init_minium(self, mini_config_dict_or_file_path=None):

        self.set_config(mini_config_dict_or_file_path)
        if not self.testconfig.report_usage:
            LazyLibs.Wechat().minium.wechatdriver.minium_log.existFlag = 1
        self.set_minium()
        self.set_native()

    def set_config(self, mini_config_dict_or_file_path=None):

        if mini_config_dict_or_file_path is None:
            self.mini_config = self.DEFAULT_MINI_CONFIG
        elif isinstance(mini_config_dict_or_file_path, dict):
            self.mini_config = LazyLibs.Wechat().miniconfig.MiniConfig(mini_config_dict_or_file_path)
        elif isinstance(mini_config_dict_or_file_path, str):
            self.mini_config = LazyLibs.Wechat().miniconfig.MiniConfig.from_file(mini_config_dict_or_file_path)
        else:
            raise ValueError("mini_config_dict_or_file_path's value must be str or dictionary")
        self.testconfig = LazyLibs.Wechat().miniconfig.MiniConfig(self.mini_config)

    def release_minium(self):

        if not self.testconfig.close_ide and self.mini:
            self.mini.shutdown()
        if self.testconfig.platform != 'ide' and not self.testconfig.close_ide:
            self.native.stop_wechat() if self.native else print("Native module has not start, there is no need to stop WeChat")
        if self.native:
            self.native.release()
        self.mini = None
        self.native = None

    @property
    def page(self):

        return self.mini.app.get_current_page()

    def get_screenshot_as_file(self, filename):
        """
        :Args:
         - filename: The full path you wish to save your screenshot to. This
           should end with a `.png` extension.
        """
        if not filename.lower().endswith('.png'):
            warnings.warn("name used for saved screenshot does not match file " "type. It should end with a `.png` extension", UserWarning)

        return self.native.screen_shot(filename)

    def get_screenshot_as_base64(self):

        filename = tempfile.mktemp(suffix=".png")
        self.get_screenshot_as_file(filename)
        raw_data = b""
        if os.path.exists(filename):
            with open(filename, "rb") as f:
                raw_data = f.read()
            os.unlink(filename)
        else:
            FileNotFoundError("No such file: {}".format(filename))
        return base64.b64encode(raw_data).decode()

    def set_minium(self):

        if self.mini is None:
            self.mini = LazyLibs.Wechat().minium.Minium(project_path=self.testconfig.project_path, test_port=self.testconfig.test_port, dev_tool_path=self.testconfig.dev_tool_path)
            if self.testconfig.enable_app_log:

                def mini_log_added(message):
                    dt = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                    message["dt"] = dt
                    self.log_message_list.append(message)

                self.mini.connection.register("App.logAdded", mini_log_added)
                self.mini.app.enable_log()
        return self.mini

    def set_native(self):

        if self.native is None:
            self.native = LazyLibs.Wechat().minium.native.get_native_driver(self.testconfig.platform, self.testconfig.device_desire)
            if self.testconfig.platform != "ide" and not self.testconfig.close_ide:
                self.set_minium()
                self.native.start_wechat()
                path = self.mini.enable_remote_debug(use_push=self.testconfig.use_push, connect_timeout=self.testconfig.remote_connect_timeout)
                if not self.testconfig.use_push:
                    self.native.connect_weapp(path)
                    self.mini.connection.wait_for(method="App.initialized")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/16
'''
import copy
import json
from ..const import Const
from ..utils.sutils import strclass
from .test_wrapper import Test
from .abstract_testcase import AbstractTestCase
from ..utils.attrs_manager import AttributeManager
from ..utils.attrs_marker import ConstAttributeMarker


class TestCaseWrapper(AttributeManager):

    SUCCESS = ConstAttributeMarker(0, "通过", alias="通过")
    FAILURE = ConstAttributeMarker(1, "失败", alias="失败")
    ERROR = ConstAttributeMarker(2, "异常", alias="异常")
    BLOCKED = ConstAttributeMarker(3, "阻塞", alias="阻塞")
    SKIPED = ConstAttributeMarker(4, "跳过", alias="skiped")
    XFAILURE = ConstAttributeMarker(5, "预期失败", alias="预期失败")
    XSUCCESS = ConstAttributeMarker(6, "预期失败却成功", alias="unexpected successes")

    CSS_CLASS_MAPS = {
        SUCCESS.value: 'test-pass',
        FAILURE.value: 'test-fail',
        ERROR.value: 'test-error',
        BLOCKED.value: 'test-blocked',
        SKIPED.value: 'test-skip',
        XFAILURE.value: 'test-xfail',
        XSUCCESS.value: 'test-unexpected-pass',
    }

    def __init__(self, test, id, result_code, message="", **kwargs):

        self.test = test
        self.id = id
        self._kwargs = kwargs
        self.set_result(result_code, message)
        self.is_abstract_testcase = isinstance(self.test, AbstractTestCase)
        # self.is_abstract_testcase = hasattr(self.test, "collect_testcases")

    @property
    def name(self):

        return self.description or self.method_name

    @property
    def method_name(self):

        if self.is_abstract_testcase:
            name = self.test.real_test_method_name
        else:
            name = self.test.id()
        return name

    @property
    def number(self):
        return self.id

    @property
    def duration(self):
        """ 返回执行用例耗时，单位秒"""

        start_time = getattr(self.test, Const.STEST_START_PERF_COUNTER, None)
        finish_time = getattr(self.test, Const.STEST_FINISH_PERF_COUNTER, None)

        if start_time is not None and finish_time is not None:
            total_time = finish_time - start_time
            return '{:f}'.format(total_time)
        else:
            return ''

    @property
    def start_time(self):

        return getattr(self.test, Const.STEST_TESTCASE_START_TIME, None)

    @property
    def exec_number(self):
        return getattr(self.test, Const.STEST_TESTCASE_EXEC_NUMBER, 0)

    @property
    def screenshot_info(self):

        k = 'screenshot_info'
        return self._kwargs.get(k, {})

    @property
    def testpoint(self):
        if self.is_abstract_testcase:
            classname = self.test.strclass()
        else:
            try:
                classname = strclass(self.test.__class__)
            except Exception:
                classname = self.test.id()

        try:
            chinese_name = self.test.__doc__.split("\n")[0].strip()
        except Exception:
            chinese_name = ""
        return (classname, chinese_name)

    @property
    def result_name(self):

        markers = self.__class__.const_attrs.values()
        for marker in markers:
            code = marker.value
            if self.__result_code == code:
                return marker.expend_kwargs["alias"]

    @property
    def result_code(self):
        return self.__result_code

    @classmethod
    def result_codes(cls):
        markers = cls.const_attrs.values()
        codes = [marker.value for marker in markers]
        return codes

    @property
    def css_class(self):

        return self.CSS_CLASS_MAPS[self.result_code]

    @property
    def extra_info(self):
        """用例额外信息，如编写者、修改者、最后修改者等"""

        tms = getattr(self.test, 'test_method_settings', {})
        info = {
            Test.AUTHOR: tms.get(Test.AUTHOR, ""),
            Test.EDITORS: tms.get(Test.EDITORS, []),
            Test.LAST_MODIFYIED_BY: tms.get(Test.LAST_MODIFYIED_BY, ""),
            Test.LAST_MODIFYIED_TIME: tms.get(Test.LAST_MODIFYIED_TIME, ""),
            "运行编号": self.exec_number,
            Test.PRIORITY: tms.get(Test.PRIORITY, ""),
            Test.DNAME: tms.get(Test.DNAME, ""),
            Test.DEPENDS: tms.get(Test.DEPENDS, ""),
            Test.GROUPS: tms.get(Test.GROUPS, ""),
        }
        return info

    @property
    def description(self):

        return self.test.shortDescription() or ''

    @property
    def output_message(self):

        return ''

    @property
    def error_message(self):

        return self.message

    @property
    def testdatas(self):
        args_mapinfo = {}
        kwargs_mapinfo = {}
        if self.is_abstract_testcase:
            spec = getattr(self.test.test_method_obj, Test.ORIGINAL_TEST_METHOD_ARGSPEC)
            runtimedatas = self.test.get_testcase_runtime_datas()
            args = runtimedatas['args']
            kwargs = runtimedatas['kwargs']
            for index, arg_value in enumerate(args):
                try:
                    arg_name = spec.args[index]
                except IndexError:
                    arg_name = ''  # 代表传入了多余的参数
                if arg_value == self.test:
                    continue
                else:
                    args_mapinfo[index] = (arg_name, arg_value)

            kwargs_mapinfo = copy.deepcopy(kwargs)

        return (args_mapinfo, kwargs_mapinfo)

    def __to_str(self, value):

        try:
            fstr = json.dumps(value, ensure_ascii=False, indent=4)
        except Exception:
            fstr = str(value)
        return fstr

    @property
    def printable_testdatas(self):

        args_mapinfo, kwargs_mapinfo = self.testdatas
        printable_args = {}
        printable_kwargs = {}
        for k, v in args_mapinfo.items():
            arg_name, arg_value = v
            printable_args[k] = (arg_name, self.__to_str(arg_value))
        for k, v in kwargs_mapinfo.items():
            printable_kwargs[k] = self.__to_str(v)
        return (printable_args, printable_kwargs)

    def set_result(self, result_code, message=""):

        if result_code not in self.result_codes():
            raise ValueError('The value range of result code is: {}'.format(' | '.join(self.result_codes())))
        self.__result_code = result_code
        self.message = message


class TestResultFormatter(object):
    def __init__(self, test_result):

        self.__test_result = test_result

    @property
    def result(self):
        return self.__test_result

    def _get_belong_project(self):

        return ""

    def to_py_json(self):

        testpoints = []
        for pointname, group_testcases in self._group_by_belong_testpoint():
            results = []
            for tc in group_testcases:
                result = dict(
                    result=dict(code=tc.result_code, name=tc.result_name, css_class=tc.css_class),
                    id=tc.id,
                    name=tc.name,
                    duration=tc.duration,
                    method_name=tc.method_name,
                    testdatas=tc.printable_testdatas,
                    number=tc.number,
                    description=tc.description,
                    output_message=tc.output_message,
                    error_message=tc.error_message,
                    screenshot_info=tc.screenshot_info,
                    extra_info=tc.extra_info,
                )
                results.append(result)
            testpoint = dict(name=pointname,
                             count=len(group_testcases),
                             pass_count=self.counts(group_testcases, TestCaseWrapper.SUCCESS),
                             fail_count=self.counts(group_testcases, TestCaseWrapper.FAILURE),
                             error_count=self.counts(group_testcases, TestCaseWrapper.ERROR),
                             block_count=self.counts(group_testcases, TestCaseWrapper.BLOCKED),
                             skip_count=self.counts(group_testcases, TestCaseWrapper.SKIPED),
                             xfail_count=self.counts(group_testcases, TestCaseWrapper.XFAILURE),
                             xpass_count=self.counts(group_testcases, TestCaseWrapper.XSUCCESS),
                             testcases=results)
            testpoints.append(testpoint)
        root = dict(project=self._get_belong_project(), testpoints=testpoints)
        return root

    def to_json_string(self):

        return json.dumps(self.to_py_json(), ensure_ascii=False)

    @property
    def testcases(self):

        testcaselist = []
        id = 1
        for test, message in self.result.errors:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.ERROR, message=message, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1
        for test, message in self.result.skipped:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.SKIPED, message=message, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1

        for test, message in self.result.failures:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.FAILURE, message=message, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1

        for test in self.result.successes:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.SUCCESS, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1

        for test, message in self.result.expectedFailures:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.XFAILURE, message=message, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1

        for test, message in self.result.unexpectedSuccesses:
            testcaselist.append(TestCaseWrapper(test, id, TestCaseWrapper.XSUCCESS, screenshot_info=self.result.screenshots.get(test, {})))
            id = id + 1
        testcaselist.sort(key=lambda one: one.exec_number)
        return testcaselist

    def _get_testpoints(self):

        testpoints = []
        for testcase in self.testcases:
            testpoint = testcase.testpoint
            clsname = testpoint[0]
            if clsname not in [tp[0] for tp in testpoints]:
                testpoints.append(testpoint)
        return testpoints

    def _group_by_belong_testpoint(self):

        groups = []
        for testpoint in self._get_testpoints():
            group_testcases = []
            for test in self.testcases:
                if testpoint[0] == test.testpoint[0]:
                    group_testcases.append(test)
            groups.append((testpoint, group_testcases))
        return groups

    @classmethod
    def counts(cls, testcases, result_code):

        return len([tc for tc in testcases if tc.result_code == result_code])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
基于unittest开发的测试框架，更友好、更灵活的编写、管理与运行测试，生成更加美观的独立单文件HTML报告。内置参数化测试数据存取方案，省去设计的烦恼，节省更多的时间，从而更快的投入到编写用例阶段

 * 使用装饰器Test来标记测试方法更加灵活
 * 支持命名测试方法且不与方法的doc string（文档字符串）冲突
 * 支持设置测试方法编写人，修改人，最后修改人以及最后一次修改时间等额外记录信息
 * 支持设置测试方法的执行顺序
 * 支持参数化功能
 * 支持数据驱动测试
 * 支持设置用例依赖
 * 内置参数化数据存取方案(使用excel（xlsx或xls格式）存取和管理维护参数化测试数据，简洁直观，易于修改维护)
 * 支持生成更加简洁美观且可作为独立文件发送的HTML测试报告
 * 支持生成jenkins junit xml 格式测试报告，用于jenkins集成
 * 支持自动查找并载入项目下的settings.py配置文件
 * 支持灵活控制测试失败自动截图并附加到测试报告中
 * 支持page object模式，内置一套易于维护的解决方案
 * 驱动管理器（DRIVER_MANAGER）更加便捷的管理打开的驱动会话
 * 对selenium、appium、minium（微信小程序自动化测试库）以及WinAppDriver（微软官方提供的一款用于做Window桌面应用程序的界面（UI）自动化测试工具）做了底层集成支持


Simple usage:

    import os

    from stest import settings
    from stest import AbstractTestCase
    from stest import Test as testcase


    class DataProvider01(object):
        def get_testdatas(self, testclass, testmethod, *args, **kwargs):

            datas = [
                {'加数1':1,'加数2':2,'预期':3},
                {'加数1':4,'加数2':5,'预期':9}
            ]
            return datas

    class DataProvider02(object):
        def get_testdatas(self, testclass, testmethod, *args, **kwargs):

            datas = [
                [{'加数1':7}, {'加数2':5}, {'预期':12}],
                [{'加数1':10}, {'加数2':5}, {'预期':15}]
            ]
            return datas

    TEST_DATA_FILE_DIRPATH = os.path.dirname(os.path.abspath(__file__))


    class CalculationTest(AbstractTestCase):

        @classmethod
        def setUpClass(cls):
            pass

        def setUp(self):
            pass

        # 自定义数据提供者 - 测试方法一个参数化示例
        @testcase(priority=1, enabled=True, data_provider=DataProvider01().get_testdatas, author='思文伟', description='整数加法测试01')
        def integer_addition_01(self, testdata):

            number_1 = testdata.get("加数1")
            number_2 = testdata.get("加数2")
            expected = testdata.get("预期")
            result = number_1 + number_2
            self.assertEqual(result, expected)

        # 自定义数据提供者 - 测试方法多个参数化示例
        @testcase(priority=2, enabled=True, data_provider=DataProvider02().get_testdatas, author='思文伟', description='整数加法测试02')
        def integer_addition_02(self, testdata_01, testdata_02, testdata_03):

            number_1 = testdata_01.get("加数1")
            number_2 = testdata_02.get("加数2")
            expected = testdata_03.get("预期")

            result = number_1 + number_2
            self.assertEqual(result, expected)

        # 不参数化示例
        @testcase(priority=3, enabled=True, author='思文伟', description='整数减法测试01')
        def integer_subtraction_01(self):

            number_1 = 21
            number_2 = 10
            result = number_1 - number_2
            expected = 11
            self.assertEqual(result, expected)

        # 使用内置的数据提供者 - 传入测试数据文件所在的目录路径
        @testcase(priority=4, enabled=True, author='思文伟', data_provider_kwargs={'data_file_dir_path':TEST_DATA_FILE_DIRPATH}, description='整数减法测试02')
        def integer_subtraction_02(self, testdata):

            number_1 = testdata.get("减数1")
            number_2 = testdata.get("减数2")
            expected = testdata.get("预期")

            result = int(number_1) - int(number_2)
            self.assertEqual(result, int(expected))

        # 使用内置的数据提供者 - 不传入测试数据文件所在的目录路径,
        # 则会检测settings.SEVEN_DATA_PROVIDER_DATA_FILE_DIR 是否设置
        # 没有设置则会使用该方法所属的测试类所在的模块目录路径作为测试数据文件的查找目录
        @testcase(priority=5, enabled=True, author='思文伟', description='整数减法测试03')
        def integer_subtraction_03(self,testdata):

            number_1 = testdata.get("减数1")
            number_2 = testdata.get("减数2")
            expected = testdata.get("预期")

            result = int(number_1) - int(number_2)
            self.assertEqual(result, int(expected))

        def tearDown(self):
            pass

        @classmethod
        def tearDownClass(cls):
            pass

    if __name__ == '__main__':
        CalculationTest.run_test()

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

__all__ = ['Test', 'AbstractTestCase', 'AbsractDataProvider', 'main', 'settings']

from .main import main
from .conf import settings
from .core.test_wrapper import Test
from .core.abstract_testcase import AbstractTestCase
from .core.abstract_data_provider import AbsractDataProvider



import copy
import operator

empty = object()


def new_method_proxy(func):
    def inner(self, *args):
        if self._wrapped is empty:
            self._setup()
        return func(self._wrapped, *args)

    return inner


class LazyObject(object):

    _wrapped = None

    def __init__(self):
        self._wrapped = empty

    __getattr__ = new_method_proxy(getattr)

    def __setattr__(self, name, value):
        if name == "_wrapped":
            # 分配给_dict__;以避免_setattr___无限递归。
            self.__dict__["_wrapped"] = value
        else:
            if self._wrapped is empty:
                self._setup()
            setattr(self._wrapped, name, value)

    def __delattr__(self, name):
        if name == "_wrapped":
            raise TypeError("can't delete _wrapped.")
        if self._wrapped is empty:
            self._setup()
        delattr(self._wrapped, name)

    def _setup(self):
        raise NotImplementedError('subclasses of LazyObject must provide a _setup() method')

    def __reduce__(self):
        if self._wrapped is empty:
            self._setup()
        return (unpickle_lazyobject, (self._wrapped, ))

    def __copy__(self):
        if self._wrapped is empty:

            return type(self)()
        else:

            return copy.copy(self._wrapped)

    def __deepcopy__(self, memo):
        if self._wrapped is empty:

            result = type(self)()
            memo[id(self)] = result
            return result
        return copy.deepcopy(self._wrapped, memo)

    __bytes__ = new_method_proxy(bytes)
    __str__ = new_method_proxy(str)
    __bool__ = new_method_proxy(bool)

    __dir__ = new_method_proxy(dir)

    __class__ = property(new_method_proxy(operator.attrgetter("__class__")))
    __eq__ = new_method_proxy(operator.eq)
    __lt__ = new_method_proxy(operator.lt)
    __gt__ = new_method_proxy(operator.gt)
    __ne__ = new_method_proxy(operator.ne)
    __hash__ = new_method_proxy(hash)

    __getitem__ = new_method_proxy(operator.getitem)
    __setitem__ = new_method_proxy(operator.setitem)
    __delitem__ = new_method_proxy(operator.delitem)
    __iter__ = new_method_proxy(iter)
    __len__ = new_method_proxy(len)
    __contains__ = new_method_proxy(operator.contains)


def unpickle_lazyobject(wrapped):
    return wrapped



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/23 16:09:22
'''


class DictToObject(object):
    def __init__(self, py_dict):
        """

        """
        if not isinstance(py_dict, dict):
            raise ValueError("Data Type Must Be dict")
        for field in py_dict:
            value = py_dict[field]
            if not isinstance(value, (dict, list, tuple)):
                setattr(self, "%s" % field, value)
            else:
                if isinstance(value, dict):
                    setattr(self, "%s" % field, self.__class__(value))
                elif isinstance(value, tuple):
                    setattr(self, "%s" % field, self.analyze_tuple(value))
                else:
                    setattr(self, "%s" % field, self.analyze_list(value))

    def analyze_list(self, array):
        for index, value in enumerate(array):
            if isinstance(value, list):
                self.analyze_list(value)
            elif isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, dict):  # if element of list is dict,converts the value of the element in the list into the object
                array[index] = self.__class__(value)
            else:
                pass
        return array

    def analyze_tuple(self, array):
        tuple_to_list = list(array)
        for index, value in enumerate(tuple_to_list):
            if isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, dict):  # if element of list is dict,converts the value of the element in the list into the object
                tuple_to_list[index] = self.__class__(value)
            else:
                pass
        return tuple(tuple_to_list)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/23 17:49:02
'''

__all__ = ['DRIVER_MANAGER', 'WIN_APP_DRIVER_HELPER']

from .driver_manager import DRIVER_MANAGER
from .win_app_driver_helper import WIN_APP_DRIVER_HELPER



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/04/06
'''
import time
from ..utils import attrs_manager
from ..utils import attrs_marker
from ..dm import DRIVER_MANAGER


class AbstractMiniumPage(attrs_manager.AttributeManager):
    """微信小程序抽象根页面"""

    WECHAT_MANAGER = attrs_marker.AttributeMarker(DRIVER_MANAGER, True, "微信小程序测试库minium管理器")

    def __init__(self, url=None, minium_config=None):
        """

        Args:
            url: 微信小程序页面url
            minium_config: 初始化配置，只初始化一次，如果在其它页面已经初始化过，则不会再次初始化。
            取值为字典、配置文件路径、None，如果为None则使用默认配置WechatMiniumProxy.DEFAULT_MINI_CONFIG
                值：
                    {
                        "platform": "ide",
                        "debug_mode": "info",
                        "close_ide": False,
                        "no_assert_capture": False,
                        "auto_relaunch": False,
                        "device_desire": {},
                        "report_usage": True,
                        "remote_connect_timeout": 180,
                        "use_push": True
                    }
        """

        self.__mini_proxy = self.WECHAT_MANAGER.open_wechat_minium(minium_config)
        self.mini = self.__mini_proxy.mini
        self.native = self.__mini_proxy.native
        self.app = self.mini.app
        self.url = url

        if self.url and isinstance(self.url, str) and self.url.strip() != "":
            self.app.redirect_to(self.url)
        self._build_elements()
        self._build_actions()
        self.init()

    @property
    def current_page(self):

        return self.app.get_current_page()

    @property
    def wechat_manager(self):
        return self.__class__.WECHAT_MANAGER

    def get_element(self, selector, inner_text=None, text_contains=None, value=None, max_timeout=20):

        return self.app.get_current_page().get_element(selector, inner_text=inner_text, text_contains=text_contains, value=value, max_timeout=max_timeout)

    def get_elements(self, selector, max_timeout=20):

        return self.app.get_current_page().get_elements(selector, max_timeout=max_timeout)

    def init(self):

        pass

    def _build_elements(self):

        self.elements = self.__class__.Elements(self)

    def _build_actions(self):

        self.actions = self.__class__.Actions(self)

    def screenshot(self, filename):

        self.self.__mini_proxy.get_screenshot_as_file(filename)
        return self

    def sleep(self, seconds):
        """seconds the length of time to sleep in seconds"""
        time.sleep(seconds)
        return self

    def raise_error(self, message=''):

        raise AssertionError(message)

    class Elements(object):
        def __init__(self, page):

            self.page = page

        def sleep(self, seconds):

            self.page.sleep(seconds)
            return self

    class Actions(object):
        def __init__(self, page):

            self.page = page

        def sleep(self, seconds):

            self.page.sleep(seconds)
            return self

        def turn_to_page(self, page_number):
            """翻页， 由具体页面实现"""

            raise NotImplementedError

        def screenshot(self, filename):

            self.page.screenshot(filename)
            return self



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/03/30 15:49:32
'''
import os
import stest
# from stest import settings
from stest import AbstractTestCase
from stest import Test as testcase


class DataProvider01(object):
    def get_testdatas(self, testclass, testmethod, *args, **kwargs):

        datas = [{'加数1': 1, '加数2': 2, '预期': 5}, {'加数1': 4, '加数2': 5, '预期': 9}]
        return datas


class DataProvider02(object):
    def get_testdatas(self, testclass, testmethod, *args, **kwargs):

        datas = [[{'加数1': 7}, {'加数2': 5}, {'预期': 12}], [{'加数1': 10}, {'加数2': 5}, {'预期': 15}]]
        return datas


TEST_DATA_FILE_DIRPATH = os.path.dirname(os.path.abspath(__file__))


class CalculationTest(AbstractTestCase):
    """数学运算测试"""
    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        pass

    @testcase(priority=1, enabled=True, screenshot=True, attach_screenshot_to_report=True, data_provider=DataProvider01().get_testdatas, author='思文伟', description='整数加法测试01')
    def integer_addition_01(self, testdata):
        """自定义数据提供者 - 测试方法一个参数化示例"""

        number_1 = testdata.get("加数1")
        number_2 = testdata.get("加数2")
        expected = testdata.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    @testcase(priority=2, enabled=True, data_provider=DataProvider02().get_testdatas, author='思文伟', description='整数加法测试02')
    def integer_addition_02(self, testdata_01, testdata_02, testdata_03):
        """自定义数据提供者 - 测试方法多个参数化示例"""

        number_1 = testdata_01.get("加数1")
        number_2 = testdata_02.get("加数2")
        expected = testdata_03.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    @testcase(priority=3, enabled=True, author='思文伟', description='整数减法测试01')
    def integer_subtraction_01(self):
        """不参数化示例"""

        number_1 = 21
        number_2 = 10
        result = number_1 - number_2
        expected = 11
        self.assertEqual(result, expected)

    @testcase(priority=4, enabled=True, author='思文伟', data_provider_kwargs={'data_file_dir_path': TEST_DATA_FILE_DIRPATH}, description='整数减法测试02')
    def integer_subtraction_02(self, testdata):
        """使用内置的数据提供者 - 传入测试数据文件所在的目录路径"""

        number_1 = testdata.get("减数1")
        number_2 = testdata.get("减数2")
        expected = testdata.get("预期")

        result = int(number_1) - int(number_2)
        self.assertEqual(result, int(expected))

    @testcase(priority=5, enabled=True, author='思文伟', description='整数减法测试03')
    def integer_subtraction_03(self, testdata):
        """使用内置的数据提供者 - 不传入测试数据文件所在的目录路径,
        则会检测settings.SEVEN_DATA_PROVIDER_DATA_FILE_DIR 是否设置
        ，没有设置则会使用该方法所属的测试类所在的模块目录路径作为测试数据文件的查找目录
        """

        number_1 = testdata.get("减数1")
        number_2 = testdata.get("减数2")
        expected = testdata.get("预期")

        result = int(number_1) - int(number_2)
        self.assertEqual(result, int(expected))

    def test_login_baidu(self):

        print("yes")

    def tearDown(self):
        pass

    @classmethod
    def tearDownClass(cls):
        pass


if __name__ == '__main__':
    # CalculationTest.run_test()
    stest.main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''

from ..utils.attrs_manager import AttributeManager


class AbsractDataProvider(AttributeManager):
    def __init__(self):
        super(AbsractDataProvider, self).__init__()

    def get_testdatas(self, test_class_name, test_method_name, *args, **kwargs):
        """当测试方法只有一个参数化时，应返回一维列表，多个参数化时返回二维列表"""

        raise NotImplementedError



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/29
'''
import os
import stest
# from stest import settings
from stest import AbstractTestCase
from stest import Test as testcase


class DataProvider01(object):
    def get_testdatas(self, testclass, testmethod, *args, **kwargs):

        datas = [{'加数1': 1, '加数2': 2, '预期': 3}, {'加数1': 4, '加数2': 5, '预期': 9}]
        return datas


class DataProvider02(object):
    def get_testdatas(self, testclass, testmethod, *args, **kwargs):

        datas = [[{'加数1': 7}, {'加数2': 5}, {'预期': 12}]]
        return datas


TEST_DATA_FILE_DIRPATH = os.path.dirname(os.path.abspath(__file__))


class DependTest(AbstractTestCase):
    """依赖设置测试"""
    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        pass

    @testcase(priority=1, enabled=True, author='思文伟', description='sqq', depends=['oss'])
    def sqq(self):

        number_1 = 21
        number_2 = 10
        result = number_1 - number_2
        expected = 11
        self.assertEqual(result, expected)

    @testcase(priority=2, enabled=True, author='思文伟', description='oss', depends=['sqq'])
    def oss(self):

        number_1 = 21
        number_2 = 10
        result = number_1 - number_2
        expected = 11
        self.assertEqual(result, expected)

    @testcase(priority=3, enabled=True, data_provider=DataProvider01().get_testdatas, author='思文伟', description='t', depends=['sqq'])
    def t(self, testdata):

        number_1 = testdata.get("加数1")
        number_2 = testdata.get("加数2")
        expected = testdata.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    @testcase(priority=4, enabled=True, data_provider=DataProvider02().get_testdatas, author='思文伟', description='one', depends=['two'])
    def one(self, testdata_01, testdata_02, testdata_03):

        number_1 = testdata_01.get("加数1")
        number_2 = testdata_02.get("加数2")
        expected = testdata_03.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    @testcase(priority=5, enabled=True, data_provider=DataProvider02().get_testdatas, author='思文伟', description='two', depends=['t3'])
    def two(self, testdata_01, testdata_02, testdata_03):

        number_1 = testdata_01.get("加数1")
        number_2 = testdata_02.get("加数2")
        expected = testdata_03.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    @testcase(priority=6, enabled=True, data_provider=DataProvider02().get_testdatas, author='思文伟', description='three', dname="t3")
    def three(self, testdata_01, testdata_02, testdata_03):

        number_1 = testdata_01.get("加数1")
        number_2 = testdata_02.get("加数2")
        expected = testdata_03.get("预期")

        result = number_1 + number_2
        self.assertEqual(result, expected)

    def tearDown(self):
        pass

    @classmethod
    def tearDownClass(cls):
        pass


if __name__ == '__main__':
    stest.main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/30 17:51:16
'''

# 内置参数化数据提供者(SevenDataProvider)读取的测试数据文件所在的目录路径，
# 不设置则自动获取测试用例所在模块的目录路径作为测试数据文件所在的目录路径，
# 内置参数化数据提供者会从该目录路径查找用例测试数据文件。
SEVEN_DATA_PROVIDER_DATA_FILE_DIR = None

# 控制测试失败是否自动截图
SCREENSHOT = False

# 截图文件存放目录，保存为文件时才会用到
# SCREENSHOT_SAVE_DIR = None

# 控制截图后是否附加到测试报告中，如果附加到报告中，则截图转base64数据附加到报告中，否则保存为文件
ATTACH_SCREENSHOT_TO_REPORT = True



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/11/15 13:53:35
'''

from .utils.attrs_manager import AttributeManager
from .utils.attrs_marker import ConstAttributeMarker


class Const(AttributeManager):

    STEST_TESTCASE_START_TIME = ConstAttributeMarker('start_time_of_stest_execution_testcase', 'testcase 实例中记录用例开始执行时间(datetime.datetime.now())的属性名')

    STEST_START_PERF_COUNTER = ConstAttributeMarker('stest_start_perf_counter', 'testcase 实例中记录用例开始执行时间的一个性能计时(time.perf_counter())的属性名')

    STEST_FINISH_PERF_COUNTER = ConstAttributeMarker('stest_finish_perf_counter', 'testcase 实例中记录用例执行完成时的一个性能计时(time.perf_counter())的属性名')

    STEST_TESTCASE_EXEC_NUMBER = ConstAttributeMarker('stest_testcase_exec_number', 'testcase 实例中记录用例执行顺序编号的属性名')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/23 16:02:25
'''
import datetime


class TimeDeltaDisplayer(datetime.timedelta):
    def human_readable(self, lang="cn"):

        if lang == "cn":
            mm, ss = divmod(self.seconds, 60)
            hh, mm = divmod(mm, 60)

            # s = "%d:%02d:%02d" % (hh, mm, ss)
            if ss > 9:
                s = "%02d" % ss
            else:
                s = "%d" % ss
            if mm > 0:
                mf = "%02d分" if mm > 9 else "%d分"
                s = (mf % mm) + s
            if hh > 0:
                s = ("%d小时" % hh) + s

            if self.days:
                s = ("%d 天, " % self.days) + s
            if self.microseconds:
                s = s + ".%06d" % self.microseconds
            s = s + "秒"
        else:
            s = str(self)
        return s



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/23
'''


class LazyLibs(object):
    class Appium(object):
        @property
        def MobileBy(self):
            from appium.webdriver.common.mobileby import MobileBy
            return MobileBy

        @property
        def webdriver(self):
            from appium import webdriver
            return webdriver

    class Selenium(object):
        @property
        def exceptions(cls):
            from selenium.common import exceptions
            return exceptions

        @property
        def ActionChains(self):
            from selenium.webdriver import ActionChains
            return ActionChains

        @property
        def Keys(self):
            from selenium.webdriver.common.keys import Keys
            return Keys

        @property
        def By(self):
            from selenium.webdriver.common.by import By
            return By

        @property
        def WebDriverWait(self):
            from selenium.webdriver.support.wait import WebDriverWait
            return WebDriverWait

        @property
        def webdriver(self):
            from selenium import webdriver
            return webdriver

        @property
        def WebElement(self):
            from selenium.webdriver.remote.webelement import WebElement
            return WebElement

        @property
        def Select(self):
            from selenium.webdriver.support.ui import Select
            return Select

    class Wechat(object):
        @property
        def minium(self):
            import minium
            return minium

        @property
        def miniconfig(self):
            from minium.framework import miniconfig
            return miniconfig



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/23 16:23:54
'''

import io
import os
import base64
from .attrs_manager import AttributeManager


class ScreenshotCapturer(AttributeManager):
    """优先使用浏览器截图，如果浏览器截图失败则执行整个屏幕截图"""
    @classmethod
    def screenshot_with_pil(cls, filename=None, region=None):
        """
        截屏返回PIL.Image对象

        Args:
         - filename: 保存截图文件的完整路径名，省略则不保存
         - region: 4整型数据构成的元祖 第1、2为起点坐标，第3、4位长宽，省略则截全屏

        Usage:
            ScreenshotCapturer.screenshot_with_pil('/screenshots/st.png')
        """
        from PIL import ImageGrab

        im = ImageGrab.grab()
        if region is not None:
            assert len(region) == 4, 'region argument must be a tuple of four ints'
            region = [int(x) for x in region]
            im = im.crop((region[0], region[1], region[2] + region[0], region[3] + region[1]))
        if filename is not None:
            im.save(filename)
        return im

    @classmethod
    def full_screenshot_with_pil(cls, file_full_path):

        img = cls.screenshot_with_pil()
        try:
            img.save(file_full_path, "png")
        except ValueError:
            return False
        except IOError:
            return False
        finally:
            del img
        return True

    @classmethod
    def screenshot_with_driver(cls, driver, file_full_path):

        try:
            return driver.get_screenshot_as_file(file_full_path)
        except Exception:
            return False

    @classmethod
    def screenshot(cls, filename, driver=None):
        """浏览器截图失败则启用屏幕截图，返回结果和截图文件路径

        Args:
         - filename: 保存截图文件的完整路径名
         - driver: 驱动实例，省略则使用PIL截图
        """

        result = False
        cls.create_dir(filename)
        if driver:
            result = cls.screenshot_with_driver(driver, filename)
        if not result:
            result = cls.full_screenshot_with_pil(filename)
        return (result, filename)

    @classmethod
    def screenshot_as_base64(cls, driver=None):

        try:
            if driver:
                return driver.get_screenshot_as_base64()
        except Exception:
            pass
        img = cls.screenshot_with_pil()
        temp = io.BytesIO()
        try:
            img.save(temp, "png")
        except ValueError:
            pass
        except IOError:
            pass
        finally:
            del img
        img_datas = temp.getvalue()
        del temp
        return base64.b64encode(img_datas).decode()

    @classmethod
    def screenshot_file_to_base64(cls, file_full_path):
        """转为base64 编码数据

        @filename 完整文件路径
        """
        raw_data = ""
        try:
            with open(file_full_path, "rb") as f:
                raw_data = f.read()
        except IOError as err:
            print(err)
        return base64.b64encode(raw_data).decode()

    @classmethod
    def create_dir(cls, path):

        target_dir = os.path.dirname(path)
        if not os.path.exists(target_dir):
            os.makedirs(target_dir)



# -*- coding:utf-8 -*-
"""
导入助手
"""

__version__ = "1.0"
__author__ = "si wen wei"

import os
from ..pylibs import python_imp as imp
import inspect


class ImportHelper(object):
    """动态导入"""
    def __init__(self):

        self.dot = "."

    def is_package(self, dirpath):

        ispkg = False
        if not os.path.isdir(dirpath):
            return ispkg

        filenames = os.listdir(dirpath)
        for fn in filenames:
            subname = inspect.getmodulename(fn)
            if subname == "__init__":
                ispkg = True
                break
        return ispkg

    def __load_package(self, dirpath):
        """
        分析包路径，依次导入路径中的包. eg: F:\\zone\\seven\\finders seven是顶层包 finders是子包
        那么将会导入这两个包, 导入后 可以通过sys.modules["seven"] sys.modules["finders"]取得包模块对象

        @param dirpath 包路径
        """

        packages = []
        while True:
            if dirpath.endswith(os.sep):
                dirpath = dirpath.rstrip(os.sep)

            if not self.is_package(dirpath):
                break

            dirname = os.path.basename(dirpath)
            dirpath = os.path.dirname(dirpath)

            if dirname:
                file_object, file_path, describer = imp.find_module(dirname, [dirpath])
                try:
                    package = imp.load_module(dirname, file_object, file_path, describer)
                    packages.append(package)
                finally:
                    if file_object:
                        file_object.close()
            else:
                break
        packages.reverse()
        return packages

    def __load_parent_module(self, modules):
        """
        handle hierarchical module names (names containing dots).
        In order to find P.M, that is, submodule M of package P,
        use find_module() and load_module() to find and load package P,
        and then use find_module() with the path argument set to P.__path__.
        When P itself has a dotted name, apply this recipe recursively. eg: seven.finders.JSONFinder

        @param modules 包类型模块对象
        """

        names = []
        for module in modules:
            names.append(module.__name__)
        parent_module_name = self.dot.join(names)
        if parent_module_name:
            the_last_module = modules[-1]
            the_last_module_name = the_last_module.__name__
            the_last_module_dir = os.path.dirname(the_last_module.__path__[-1])

            file_object, file_path, describer = imp.find_module(the_last_module_name, [the_last_module_dir])
            try:
                module = imp.load_module(parent_module_name, file_object, file_path, describer)
                modules.append(module)
            finally:
                if file_object:
                    file_object.close()
        return parent_module_name

    def load_modules(self, dirpath):
        """
        动态导入目录下的所有py文件模块(__init__.py除外)

        @param dirpath 模块所在的目录路径
        """

        if not os.path.isdir(dirpath):
            raise EnvironmentError('%s is not a directory' % dirpath)

        parent_module_name = self.__load_parent_module(self.__load_package(dirpath))
        names = os.listdir(dirpath)
        modules = []

        for name in names:
            if os.path.isfile(os.path.join(dirpath, name)):
                if name.endswith('.py') and name != '__init__.py':

                    module_name = name[:-3]
                    file_object, file_path, describer = imp.find_module(module_name, [dirpath])
                    try:
                        if parent_module_name:
                            hierarchical_module_name = self.dot.join([parent_module_name, module_name])
                        else:
                            hierarchical_module_name = module_name

                        module = imp.load_module(hierarchical_module_name, file_object, file_path, describer)
                        modules.append(module)
                    finally:
                        if file_object:
                            file_object.close()
        return modules

    def load_module(self, module_file_path):
        """
        动态导入目录下的所有py文件模块(__init__.py除外)

        @param dirpath 模块所在的目录路径
        """

        if not os.path.isfile(module_file_path):
            raise EnvironmentError('%s is not a file' % module_file_path)

        dirpath = os.path.dirname(module_file_path)
        parent_module_name = self.__load_parent_module(self.__load_package(dirpath))
        name = os.path.basename(module_file_path)
        module = None

        if os.path.isfile(os.path.join(dirpath, name)):
            if name.endswith('.py') and name != '__init__.py':

                module_name = name[:-3]
                file_object, file_path, describer = imp.find_module(module_name, [dirpath])
                try:
                    if parent_module_name:
                        hierarchical_module_name = self.dot.join([parent_module_name, module_name])
                    else:
                        hierarchical_module_name = module_name

                    module = imp.load_module(hierarchical_module_name, file_object, file_path, describer)
                finally:
                    if file_object:
                        file_object.close()
        return module



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/04/22 18:06:23
'''

from ..core.errors import ConstAttributeException
from .attrs_marker import AttributeMarker


class AttributeMetaclass(type):

    _CLASS_ATTRIBUTES = "_CLASS_ATTRIBUTES"  # 禁止修改的类属性和类实例属性名

    def __new__(cls, name, bases, attrs):

        attributes = dict()

        for base in bases:
            if hasattr(base, cls._CLASS_ATTRIBUTES):
                attributes.update(base._CLASS_ATTRIBUTES)

        for attribute, value in attrs.items():
            if isinstance(value, AttributeMarker):
                attributes[attribute] = value

        for attribute in attributes.keys():
            if attribute in attrs.keys():
                # attrs.pop(attribute)
                attrs[attribute] = attributes[attribute].value
        attrs[cls._CLASS_ATTRIBUTES] = attributes

        return type.__new__(cls, name, bases, attrs)

    def __init__(cls, name, bases, attrs):

        return type.__init__(cls, name, bases, attrs)

    def __setattr__(cls, name, value):

        if name in cls.const_attrs.keys():
            raise ConstAttributeException("Unable to modify constant attribute(%s)" % name)
        type.__setattr__(cls, name, value)

    @property
    def const_attrs(cls):
        """禁止修改的属性"""

        attrs = {}
        for attr, marker in cls._CLASS_ATTRIBUTES.items():

            if marker.final:
                attrs[attr] = marker
        return attrs

    def get_attribute_marker(cls, attribute_name):

        if attribute_name in cls.const_attrs.keys():
            return cls.const_attrs[attribute_name]
        else:
            raise ConstAttributeException("'%s' class has no attribute '%s'" % (cls.__name__, attribute_name))


class AttributeManager(object, metaclass=AttributeMetaclass):
    def __setattr__(self, name, value):

        if name in self.__class__.const_attrs.keys():
            raise ConstAttributeException("Unable to modify constant attribute(%s)" % name)
        object.__setattr__(self, name, value)

    def get_attr_marker(self, attribute_name):

        return self.__class__.get_attribute_marker(attribute_name)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''

import sys
import xlrd
from ..utils import sutils
from .attrs_marker import ConstAttributeMarker
from .attrs_manager import AttributeManager
import io
sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')


class TestCaseBlock(AttributeManager):
    """
    测试用例块
    块中每行数据定义如下:
    所有行中的第一列是标记列
    第一行:    用例名称信息(标记列的下一列是用例名称列，之后是用例别名列)
    第二行:    用例数据标题
    第三行 开始 每一行都是一组完整的测试数据直至遇见空行或者下一个数据块
    """

    UTF_8 = ConstAttributeMarker("UTF-8", "UTF-8字符编码")
    EMPTY_STRING = ConstAttributeMarker("", "空字符串")
    XL_CELL_EMPTY = ConstAttributeMarker(0, "Python value：empty string ''")
    XL_CELL_TEXT = ConstAttributeMarker(1, "Python value：a Unicode string")
    XL_CELL_NUMBER = ConstAttributeMarker(2, "Python value：float")
    XL_CELL_DATE = ConstAttributeMarker(3, "Python value：float")
    XL_CELL_BOOLEAN = ConstAttributeMarker(4, "Python value：int; 1 means TRUE, 0 means FALSE")
    XL_CELL_ERROR = ConstAttributeMarker(5, "Python value：int representing internal Excel codes; for a text representation, refer to the supplied dictionary error_text_from_code")
    XL_CELL_BLANK = ConstAttributeMarker(6, "Python value：empty string ''. Note: this type will appear only when open_workbook(..., formatting_info=True) is used.")

    NAME_ROW_INDEX = ConstAttributeMarker(0, "例块中名称行索引")
    TITLE_ROW_INDEX = ConstAttributeMarker(1, "用例块中数据标题索引")
    MIN_NUMBER_OF_ROWS = ConstAttributeMarker(3, "用例区域至少需要的行数")

    def __init__(self, flag_column_index=0):
        """
        Args:
            flag_column_index 用例区块内分隔符列索引
        """

        self._rows = []  # [(row index in excel file, row), ...]
        self._flag_column_index = flag_column_index

    @property
    def rows(self):

        return self._rows

    def is_valid(self):
        """校验用例数据块是否符合要求（行数最少包含3行 用例名称 行 , 用例数据标题 行 ,  用例数据标题 行[可以有多行]）"""
        return False if len(self.rows) < self.MIN_NUMBER_OF_ROWS else True

    def is_integer(self, number):

        return number % 1 == 0.0 or number % 1 == 0

    def add_block_rows(self, *block_rows):

        for block_row in block_rows:
            self._rows.append(block_row)

    def get_name_row(self):

        return self.rows[self.NAME_ROW_INDEX][1]

    def _get_name_row_values(self):

        values = []
        for cell in self.get_name_row():

            ctype = cell.ctype
            value = cell.value

            if ctype == self.XL_CELL_TEXT and value.strip() != self.EMPTY_STRING:
                values.append(value)
            elif ctype == self.XL_CELL_NUMBER:
                if self.is_integer(value):
                    values.append(str(int(value)))
                else:
                    values.append(str(value))
            else:
                # raise Warning("用例名称单元格类型必须是文本且不能为空")
                pass
        return values

    @property
    def name_column_index(self):

        return self._flag_column_index + 1

    @property
    def alias_column_index(self):

        return self.name_column_index + 1

    @property
    def testcase_name(self):

        cell_values = self._get_name_row_values()
        return cell_values[self.name_column_index]

    @property
    def testcase_alias(self):

        cell_values = self._get_name_row_values()
        return cell_values[self.alias_column_index]

    def is_empty_string(self, value):

        return value.strip() == self.EMPTY_STRING

    def _do_nothing(self, *objs):
        return objs

    def get_data_titles(self):

        titles = []

        for index, cell in enumerate((self.rows[self.TITLE_ROW_INDEX][1])):
            if index == self._flag_column_index:
                continue

            if cell.ctype == self.XL_CELL_TEXT and not self.is_empty_string(cell.value):
                titles.append((index, cell.value))
            else:
                val = cell.value.decode(self.UTF_8) if isinstance(cell.value, bytes) else cell.value
                self._do_nothing(val)
                # raise Warning("用例名称单元格类型必须是文本类型, 单元格类型(%s)=%s" % (cell.ctype, val))
                break
        return titles

    def get_testdata(self):

        all_row_data = []
        for row_index, item in enumerate(self.rows):
            index_in_excel_file, row = item
            if row_index + 1 >= self.MIN_NUMBER_OF_ROWS:
                one_row_data = []
                for title_cell_index, title in self.get_data_titles():
                    value_cell = row[title_cell_index]
                    if value_cell.ctype == self.XL_CELL_TEXT:
                        value = value_cell.value
                    elif value_cell.ctype == self.XL_CELL_EMPTY or value_cell.ctype == self.XL_CELL_BLANK:
                        value = value_cell.value
                    else:
                        raise Warning("用例(%s)数据单元格(%s行%s列)类型必须是文本类型" % (self.testcase_name, index_in_excel_file + 1, title_cell_index + 1))
                    one_row_data.append({title: value})
                all_row_data.append(one_row_data)
        return all_row_data


class TestCaseData(object):
    def __init__(self, name, alias=""):

        self.name = name
        self.alias = alias
        self.datas = []


class TestCaseExcelFileReader(AttributeManager):
    """ 用例数据文件读取器
    Args:
        filename: 要打开的excel电子表格文件的路径。
        testcase_block_separators: 用例分割标记
        testcase_block_separators_column_index: 用例分割标记列索引
        sheet_index_or_name: 工作表索引或者名称
    """

    UTF_8 = ConstAttributeMarker("UTF-8", "UTF-8字符编码")
    EMPTY_STRING = ConstAttributeMarker("", "空字符串")
    XL_CELL_EMPTY = ConstAttributeMarker(0, "Python value：empty string ''")
    XL_CELL_TEXT = ConstAttributeMarker(1, "Python value：a Unicode string")
    XL_CELL_NUMBER = ConstAttributeMarker(2, "Python value：float")
    XL_CELL_DATE = ConstAttributeMarker(3, "Python value：float")
    XL_CELL_BOOLEAN = ConstAttributeMarker(4, "Python value：int; 1 means TRUE, 0 means FALSE")
    XL_CELL_ERROR = ConstAttributeMarker(5, "Python value：int representing internal Excel codes; for a text representation, refer to the supplied dictionary error_text_from_code")
    XL_CELL_BLANK = ConstAttributeMarker(6, "Python value：empty string ''. Note: this type will appear only when open_workbook(..., formatting_info=True) is used.")
    DEFAULT_SHEET_INDEX = ConstAttributeMarker(0, "默认取excel的工作表索引")
    DEFAULT_TESTCASE_BLOCK_SEPARATORS = ConstAttributeMarker("用例名称", "默认用例分割标记")

    def __init__(self, filepath, testcase_block_separators="用例名称", testcase_block_separators_column_index=0, sheet_index_or_name=0):
        """ 用例数据文件读取器
        Args:
            filename: 要打开的excel电子表格文件的路径。
            testcase_block_separators: 用例分割标记
            testcase_block_separators_column_index: 用例分割标记列索引
            sheet_index_or_name: 工作表索引或者名称
        """

        self.filepath = filepath
        self.testcase_block_separators = testcase_block_separators if (isinstance(testcase_block_separators, str) and testcase_block_separators) else self.DEFAULT_TESTCASE_BLOCK_SEPARATORS
        self.testcase_block_separators_column_index = testcase_block_separators_column_index if sutils.is_positive_integer(testcase_block_separators_column_index) else 0
        self.sheet_index_or_name = sheet_index_or_name if sutils.is_positive_integer(sheet_index_or_name) else self.DEFAULT_SHEET_INDEX

        self.open()
        self.select_sheet(self.sheet_index_or_name)

    def open(self):

        self.workbook = xlrd.open_workbook(self.filepath)

    @property
    def sheet(self):

        attr_name = "_sheet"
        if not hasattr(self, attr_name):
            raise AttributeError('{} has no attributes: {}'.format(self, attr_name))
        return self._sheet

    def debug(self):

        testcases = self.load_testcase_data()
        print(len(testcases))
        tc = testcases[0]
        for row in tc.datas:
            line = []
            for cell in row:
                for key in cell:
                    line.append(key + " " + str(cell[key]))
            print(" | ".join(line))
        r1 = tc.datas[0]
        print(r1[0].get("路径"))

    def row_len(self, row_index):

        return self._sheet.row_len(row_index)

    def select_sheet(self, sheet_index_or_name):

        if isinstance(sheet_index_or_name, str):
            self.sheet_index_or_name = sheet_index_or_name
            self._sheet = self.workbook.sheet_by_name(sheet_index_or_name)
        elif isinstance(sheet_index_or_name, int):
            self.sheet_index_or_name = sheet_index_or_name
            self._sheet = self.workbook.sheet_by_index(sheet_index_or_name)
        else:
            raise Warning("传入的工作表名称必须是字符串类型，索引必须是整形数值")
        return self.sheet

    def is_blank_cell(self, cell):

        return cell.ctype == self.XL_CELL_EMPTY or cell.ctype == self.XL_CELL_BLANK

    def is_blank_row(self, row_or_index):

        is_blank = True
        if isinstance(row_or_index, int):
            cells = self._sheet.row(row_or_index)
        else:
            cells = row_or_index
        for cell in cells:
            if not self.is_blank_cell(cell):
                is_blank = False
                break
        return is_blank

    def get_row_indexes(self):

        return range(self._sheet.nrows)

    def get_last_row_index(self):

        return max(self.get_row_indexes())

    def get_testcase_blocks(self):
        """解析并获取用例文件中的用例块区域"""

        zero = 0
        block_start_row_index_list = []
        testcase_block_list = []
        for row_index in self.get_row_indexes():
            if (self.row_len(row_index) == zero):
                continue
            first_cell = self._sheet.cell(row_index, self.testcase_block_separators_column_index)
            if first_cell.ctype == self.XL_CELL_TEXT and first_cell.value == self.testcase_block_separators:
                block_start_row_index_list.append(row_index)

        count = len(block_start_row_index_list)
        for i in range(count):
            testcase_block = TestCaseBlock(self.testcase_block_separators_column_index)
            start_row_index = block_start_row_index_list[i]

            next = i + 1  # 下一个元素索引
            if next >= count:
                end_row_index = self.get_last_row_index()
            else:
                block_other_row_indexs = []
                for r_index in self.get_row_indexes():
                    if r_index >= start_row_index and r_index < block_start_row_index_list[next]:
                        block_other_row_indexs.append(r_index)
                end_row_index = max(block_other_row_indexs)

            for this_row_index in self.get_row_indexes():
                if this_row_index >= start_row_index and this_row_index <= end_row_index:
                    one_row = self._sheet.row(this_row_index)
                    if not self.is_blank_row(one_row):
                        testcase_block.add_block_rows((this_row_index, one_row))
            testcase_block_list.append(testcase_block)
        return testcase_block_list

    def load_testcase_data(self):

        testcases = []
        for testcase_block in self.get_testcase_blocks():

            testcase = TestCaseData(testcase_block.testcase_name, testcase_block.testcase_alias)
            testcase.datas = testcase_block.get_testdata()

            if testcase.name.strip() != self.EMPTY_STRING:
                testcases.append(testcase)
        return testcases



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2022/03/18
'''

from ..utils import sutils
from ..core.errors import NoOpenDriver


class DriverCache(object):
    def __init__(self, error_msg="No current driver"):

        self._error_msg = error_msg
        self._init_items()

    def _init_items(self):

        self._all_drivers = []
        self._current_driver = None
        self._closed_drivers = set()
        self._alias_index_maps = sutils.StringKeyDict()
        self._default_quit_method = 'quit'
        self._quit_methods = sutils.StringKeyDict()

    @property
    def default_quit_method(self):

        return self._default_quit_method

    @default_quit_method.setter
    def default_quit_method(self, value):

        if isinstance(value, str):
            self._default_quit_method = value
        else:
            raise TypeError('pass quit method name of driver must be string')

    @property
    def error_msg(self):

        return self._error_msg

    @error_msg.setter
    def error_msg(self, value):

        if isinstance(value, str):
            self._error_msg = value
        else:
            raise TypeError('value must be string')

    @property
    def current_driver(self):

        if not self:
            raise NoOpenDriver(self.error_msg)
        return self._current_driver

    @property
    def current_index(self):

        if not self:
            return None
        for index, driver in enumerate(self._all_drivers):
            if driver is self._current_driver:
                return index + 1

    def register_driver(self, driver, alias=None, quit_method=None):
        """注册driver，并返回注册成功后，它在缓存中的位置索引(索引号从1开始)

        Args:
            driver: 驱动对象

            alias: 驱动别名，忽略大小写和空格

            quit_method: 退出驱动的方法名，调用关闭方法时将调用驱动实例的该方法退出，忽略则调用默认退出方法，如果驱动实例没有该方法则抛异常
        """
        if driver is None:
            raise ValueError('invalid driver: {}'.format(driver))
        self._current_driver = driver
        self._all_drivers.append(driver)
        index = len(self._all_drivers)
        if sutils.is_string(alias):
            self._alias_index_maps[alias] = index
        if sutils.is_string(quit_method):
            self._quit_methods[str(index)] = quit_method
        return index

    def switch_driver(self, alias_or_index):
        """通过别名或索引切换到对应的driver"""
        self._current_driver = self._get_driver(alias_or_index)
        return self._current_driver

    def _get_driver(self, alias_or_index=None):
        """通过别名或索引获取缓存中驱动对象

        Args:
            alias_or_index: 如果传入None则返回当前激活的驱动对象，否则根据指定的别名或索引查找并返回
        """
        if alias_or_index is None:
            if not self:
                raise RuntimeError(self.error_msg)
            return self._current_driver
        try:
            index = self._resolve_alias_or_index(alias_or_index)
        except ValueError:
            raise RuntimeError("Non-existing index or alias '%s'." % alias_or_index)
        return self._all_drivers[index - 1]

    @property
    def drivers(self):

        return self._all_drivers

    @property
    def active_drivers(self):

        open_drivers = []
        for driver in self._all_drivers:
            if driver not in self._closed_drivers:
                open_drivers.append(driver)
        return open_drivers

    def _call_method(self, driver, method_name):

        return getattr(driver, method_name)()

    def close_driver(self):

        if self._current_driver:
            driver = self._current_driver
            method = self._quit_methods.get(str(self.current_index), self.default_quit_method)
            self._call_method(self._current_driver, method)
            self._current_driver = None
            if driver not in self._closed_drivers:
                self._closed_drivers.add(driver)

    def close_all_drivers(self):

        for index, driver in enumerate(self._all_drivers):
            if driver not in self._closed_drivers:
                self._call_method(driver, self._quit_methods.get(str(index + 1), self.default_quit_method))
        self.clear_empty_cache()
        return self._current_driver

    def clear_empty_cache(self):

        self._init_items()

    def get_index(self, alias_or_index):

        try:
            index = self._resolve_alias_or_index(alias_or_index)
        except ValueError:
            return None
        try:
            driver = self._get_driver(alias_or_index)
        except RuntimeError:
            return None
        return None if driver in self._closed_drivers else index

    def _resolve_alias_or_index(self, alias_or_index):
        try:
            return self._resolve_alias(alias_or_index)
        except ValueError:
            return self._resolve_index(alias_or_index)

    def _resolve_alias(self, alias):
        if sutils.is_string(alias):
            try:
                return self._alias_index_maps[alias]
            except KeyError:
                pass
        raise ValueError

    def _resolve_index(self, index):
        try:
            index = int(index)
        except TypeError:
            raise ValueError
        if not 0 < index <= len(self._all_drivers):
            raise ValueError
        return index

    def __nonzero__(self):

        return self._current_driver is not None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/04/22 18:12:55
'''


class StestException(Exception):
    pass


class ImproperlyConfigured(StestException):
    """stest is somehow improperly configured"""
    pass


class NoOpenDriver(StestException):
    pass


class AttributeMarkerException(StestException):
    pass


class ConstAttributeException(StestException):
    pass


class WindowNotFound(StestException):
    pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/11/12 10:25:11
'''
from xml.etree import ElementTree as ET
from xml.sax import saxutils

from ..core.result_formatter import TestCaseWrapper


class JenkinsXMLElementTags(object):

    testsuites = 'testsuites'
    testsuite = 'testsuite'
    testcase = 'testcase'
    skipped = 'skipped'
    error = 'error'
    failure = 'failure'
    system_out = 'system-out'


class JenkinsJunitXMLReportTemplate(object):
    TAGS = JenkinsXMLElementTags

    def __init__(self, testpoints, test_suite_name='stest'):

        self.testpoints = testpoints
        self.test_suite_name = test_suite_name

        self.el_testsuites = ET.Element(self.TAGS.testsuites)

    def __build_testsuite_element(self, name, tests, errors, failures, skipped, **others):
        """

        Args:
            - tests: The total number of tests in the suite, required.

            - disabled: the total number of disabled tests in the suite. optional. not supported by maven surefire.

            - errors: The total number of tests in the suite that errored. An errored test is one that had an unanticipated problem,
                    for example an unchecked throwable; or a problem with the implementation of the test. optional

            - failures: The total number of tests in the suite that failed. A failure is a test which the code has explicitly failed
                    by using the mechanisms for that purpose. e.g., via an assertEquals. optional

            - hostname: Host on which the tests were executed. 'localhost' should be used if the hostname cannot be determined. optional.
                        not supported by maven surefire.

            - id: Starts at 0 for the first testsuite and is incremented by 1 for each following testsuite. optional. not supported by maven surefire.

            - package: Derived from testsuite/@name in the non-aggregated documents. optional. not supported by maven surefire.

            - skipped: The total number of skipped tests. optional

            - time: Time taken (in seconds) to execute the tests in the suite. optional

            - timestamp: when the test was executed in ISO 8601 format (2014-01-21T16:17:18). Timezone may not be specified. optional.
                        not supported by maven surefire.
        """

        tests = str(tests)
        errors = str(errors)
        failures = str(failures)
        skipped = str(skipped)
        el_testsuite = ET.Element(self.TAGS.testsuite, name=saxutils.quoteattr(name), tests=tests, errors=errors, failures=failures, skipped=skipped, **others)
        return el_testsuite

    def __build_testcase_element(self, name, classname, time, **result):

        tc = ET.Element(self.TAGS.testcase, name=saxutils.quoteattr(name), classname=saxutils.quoteattr(classname), time=time)

        tagname = None
        if self.TAGS.skipped in result:
            tagname = self.TAGS.skipped
            summary, details = result[self.TAGS.skipped]
        elif self.TAGS.failure in result:
            tagname = self.TAGS.failure
            summary, details = result[self.TAGS.failure]
        elif self.TAGS.error in result:
            tagname = self.TAGS.error
            summary, details = result[self.TAGS.error]
        else:
            summary = ''
            details = ''
        if tagname:
            child = ET.Element(tagname, message=saxutils.quoteattr(summary))
            child.text = saxutils.escape(details)
            tc.append(child)

        # <system-out>STDOUT text</system-out>
        if self.TAGS.system_out in result:
            el_sysout = ET.Element(self.TAGS.system_out)
            el_sysout.text = saxutils.escape(result.get(self.TAGS.system_out, ""))
        return tc

    def to_xml(self):

        tests = 0
        error_count = 0
        skip_count = 0
        fail_count = 0

        el_testcases = []
        for testpoint in self.testpoints:
            tests = tests + testpoint['count']
            error_count = error_count + testpoint['error_count']
            skip_count = skip_count + testpoint['skip_count'] + testpoint['block_count']
            fail_count = fail_count + testpoint['fail_count'] + testpoint['xpass_count']
            classname = testpoint['name'][0]
            testcases = testpoint['testcases']

            for testcase in testcases:
                name = testcase['name']
                method = testcase['method_name']
                duration = testcase['duration']
                output_message = testcase['output_message']
                error_message = testcase['error_message']
                summary_and_details = [error_message, error_message]

                rinfo = testcase['result']
                rcode = rinfo['code']
                case_result = {}
                if output_message:
                    case_result[self.TAGS.system_out] = output_message
                if rcode in [TestCaseWrapper.FAILURE, TestCaseWrapper.XSUCCESS]:
                    case_result[self.TAGS.failure] = summary_and_details
                elif rcode in [TestCaseWrapper.SKIPED, TestCaseWrapper.BLOCKED]:
                    case_result[self.TAGS.skipped] = summary_and_details
                elif rcode in [TestCaseWrapper.ERROR]:
                    case_result[self.TAGS.error] = summary_and_details
                el_tc = self.__build_testcase_element('{} - {}'.format(method, name), classname, duration, **case_result)
                el_testcases.append(el_tc)

        suite = self.__build_testsuite_element(self.test_suite_name, tests, error_count, fail_count, skip_count)
        for one in el_testcases:
            suite.append(one)
        self.el_testsuites.append(suite)
        return ET.tostring(self.el_testsuites, encoding="utf-8", xml_declaration=True)

    def save_as_file(self, full_file_path):

        with open(full_file_path, "w", encoding="utf-8") as f:
            f.write(self.to_xml().decode("utf-8"))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/17
'''
from unittest.suite import TestSuite
from unittest.runner import TextTestRunner
from ..const import Const
from .depends import DependsManager
from .seven_result import SevenTestResult


class SevenTestRunner(TextTestRunner):

    resultclass = SevenTestResult

    def __init__(self, stream=None, descriptions=True, verbosity=1, failfast=False, buffer=False, resultclass=None, warnings=None, *, tb_locals=False, depend_manager=None):
        super().__init__(stream=stream, descriptions=descriptions, verbosity=verbosity, failfast=failfast, buffer=buffer, resultclass=resultclass, warnings=warnings, tb_locals=tb_locals)
        self.depend_manager = DependsManager() if depend_manager is None else depend_manager

    def _makeResult(self):
        results = super()._makeResult()
        self.depend_manager.results = results
        results.depend_manager = self.depend_manager
        return results

    def run(self, test):
        def _issuite(tc):
            "A crude way to tell apart testcases and suites with duck-typing"
            try:
                iter(tc)
            except TypeError:
                return False
            return True

        def all_in_one(tc):

            alltests = []
            if _issuite(tc):
                for one in tc:
                    if not _issuite(one):
                        alltests.append(one)
                    else:
                        alltests.extend(all_in_one(one))
            else:
                alltests.append(tc)
            return alltests

        def set_exec_number_to_testcase(testcases):

            for i, t in enumerate(testcases):
                setattr(t, Const.STEST_TESTCASE_EXEC_NUMBER, i + 1)

        testlist = all_in_one(test)
        self.depend_manager.tests = testlist
        suite = self.depend_manager.sorted_tests(suiteclass=TestSuite)
        set_exec_number_to_testcase(suite)
        return super().run(suite)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/15
'''
import time
import datetime
import traceback
from unittest.runner import TextTestResult

from ..const import Const
from ..conf import settings
from .test_wrapper import Test
from ..dm import DRIVER_MANAGER
from ..utils.screenshot_capturer import ScreenshotCapturer


class SevenTestResult(TextTestResult):
    def __init__(self, stream, descriptions, verbosity, depend_manager=None):
        super().__init__(stream, descriptions, verbosity)
        self.successes = []
        self.screenshots = {}  # 存放测试失败截图的base64数据
        self.depend_manager = depend_manager

    def _screenshot(self, test):

        try:
            driver = DRIVER_MANAGER.driver
        except Exception:
            driver = None

        test_method_settings = getattr(test, 'test_method_settings', {})
        screenshot = test_method_settings.get(Test.SCREENSHOT, getattr(settings, Test.SCREENSHOT.upper(), False))
        attach = test_method_settings.get(Test.ATTACH_SCREENSHOT_TO_REPORT, getattr(settings, Test.ATTACH_SCREENSHOT_TO_REPORT.upper(), False))
        capturerclass = ScreenshotCapturer
        if screenshot:
            message = ''
            try:
                data = capturerclass().screenshot_as_base64(driver)
            except Exception:
                success = False
                message = traceback.format_exc()
                data = ''
            else:
                success = True
            self.screenshots[test] = {Test.SCREENSHOT: screenshot, Test.ATTACH_SCREENSHOT_TO_REPORT: attach, 'result': success, 'base64data': data, 'message': message}

    def startTest(self, test):

        starttime = datetime.datetime.now()
        scounter = time.perf_counter()
        setattr(test, Const.STEST_START_PERF_COUNTER, scounter)
        setattr(test, Const.STEST_TESTCASE_START_TIME, starttime)
        return super().startTest(test)

    def stopTest(self, test):

        rv = super().stopTest(test)
        fcounter = time.perf_counter()
        setattr(test, Const.STEST_FINISH_PERF_COUNTER, fcounter)
        return rv

    def addSuccess(self, test):

        return_value = super().addSuccess(test)
        self.successes.append(test)
        return return_value

    def addError(self, test, err):
        rv = super().addError(test, err)
        self._screenshot(test)
        return rv

    def addFailure(self, test, err):
        rv = super().addFailure(test, err)
        self._screenshot(test)
        return rv

    def addUnexpectedSuccess(self, test):
        rv = super().addUnexpectedSuccess(test)
        self._screenshot(test)
        return rv

    def getDescription(self, test):

        short_desc = test.shortDescription()
        if self.descriptions and short_desc:
            return '\n'.join((str(test), short_desc))
        else:
            return str(test)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/08/19
'''

import subprocess


class WinAppDriverHelper(object):
    def __init__(self):
        self.process = None
        self.executable_path = None

    def startup_winappdriver(self, executable_path, output_stream=None, auto_close_output_stream=False):
        """启动WinAppDriver.exe"""

        self.executable_path = executable_path
        try:
            # output_stream = open(os.devnull, 'w')
            self.process = subprocess.Popen([executable_path], stdout=output_stream)
        except Exception:
            self.process = None
        finally:
            if auto_close_output_stream and output_stream is not None:
                output_stream.close()

    def shutdown_winappdriver(self):
        """关闭WinAppDriver.exe"""

        from psutil import Process, NoSuchProcess
        try:
            process = Process(self.process.pid)
            for pro in process.children(recursive=True):
                pro.kill()
                pro.wait()
            self.process.kill()
            self.process.wait()
            self.process = None
        except (NoSuchProcess, AttributeError):
            subprocess.call("C:/Windows/system32/taskkill.exe /f /im WinAppDriver.exe", shell=False)
            self.process = None


WIN_APP_DRIVER_HELPER = WinAppDriverHelper()
